<?php
/**
 * WP File Download
 *
 * @package WP File Download
 * @author  Joomunited
 * @version 1.0
 */

// no direct access
defined('ABSPATH') || die();

use Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Model;
use Joomunited\WPFramework\v1_0_5\Utilities;

/**
 * Class WpfdAddonWoocommerce
 */
class WpfdAddonWoocommerce
{
    const ONLY_WOO_PERMISSION = 'only_woo_permission';
    const BOTH_PERMISSION = 'both_woo_and_wpfd_permission';
    const STATISTICS_TYPE = 'wc_order';

    /**
     * WpfdAddonWoocommerce constructor.
     */
    public function __construct()
    {
        if (!$this->isWoo()) {
            return false;
        }
        add_filter('wpfd_locate_template', array($this, 'wpfdaLocateTemplate'), 10, 2);

        if (is_admin()) {
            // Load scripts/styles
            add_action('admin_enqueue_scripts', array($this, 'addAdminScripts'));
            // Inject data tab
            add_filter('woocommerce_product_data_tabs', array($this, 'addProductTab'));
            // Inject data panel
            add_action('woocommerce_product_data_panels', array($this, 'addProductDataPanel'));
            // Save product action
            add_action('woocommerce_process_product_meta', array($this, 'wpfdaWooAddFilesSave'));
            // Show woocommerce icon in backend file list
            add_filter('wpfd_admin_files', array($this, 'wpfdaShowLinkedProduct')); // Table layout
            add_filter('wpfd_file_params', array($this, 'wpfdaShowLinkedProduct')); // Alternate layout
            // Add filter to check file has linked to a product
            add_filter('wpfd_addon_has_products', array($this, 'wpfdaFileHasProducts'), 10, 2);
            // Add filter to add product data to file row
            add_filter('wpfd_admin_file_row_data', array($this, 'wpfdaFileRowData'), 10, 2);
            // Add permission option to file fields
            add_action('wpfd_admin_after_file_title_field', array($this, 'wpfdaAddWocommercePermissionField'));
            // Save permission field
            add_filter('wpfd_before_save_file_metadata', array($this, 'wpfdaSaveWooPermissionField'), 10, 2);
            // Statistics Option
            add_filter('wpfd_statistics_filter_by', function ($options) {
                $options['file_orders'] = esc_html__('File orders', 'wpfdAddon');

                return $options;
            });
            add_filter('wpfd_statistics_filter_by_placeholders', function ($placeholders) {
                $placeholders['file_orders'] = esc_html__('Select files', 'wpfdAddon');

                return $placeholders;
            });
            add_filter('wpfd_statistics_get_selection_value', function ($options, $selection) {
                if ($selection === 'file_orders') {
                    global $wpdb;
                    // Get linked files

                    // Get orders
                    $result    = $wpdb->get_col(
                        $wpdb->prepare(
                            'SELECT meta_value FROM ' . $wpdb->postmeta . ' WHERE meta_key = %s;',
                            '_wpfd_order_data'
                        )
                    );
                    $downloads = array();

                    if (is_array($result) && is_countable($result) && count($result)) {
                        foreach ($result as $order) {
                            $downloads = array_merge(maybe_unserialize($order), $downloads);
                        }
                    }

                    if (is_array($downloads) && is_countable($downloads) && count($downloads)) {
                        foreach ($downloads as $download) {
                            $orderId = wc_get_order_id_by_order_key($download['order_key']);
                            $order   = wc_get_order($orderId);
                            if ($order->has_status('completed')) {
                                // OK, add this to options array
                                $options[$download['file_data']['id']] = $download['file_data']['name'];
                            }
                        }
                    }
                }

                return array_unique($options);
            }, 10, 2);
            add_filter('wpfd_statistics_get_selection_arguments', function ($arguments, $selection, $selection_value) {
                if ($selection === 'file_orders') {
                    add_filter('wpfd_statistics_download_count_text', function ($text) {
                        return esc_html__('Order Count', 'wpfdAddon');
                    });

                    global $wpdb;
                    // Get orders
                    $result    = $wpdb->get_col(
                        $wpdb->prepare(
                            'SELECT meta_value FROM ' . $wpdb->postmeta . ' WHERE meta_key = %s;',
                            '_wpfd_order_data'
                        )
                    );
                    $downloads = array();

                    if (is_array($result) && is_countable($result) && count($result)) {
                        foreach ($result as $order) {
                            $downloads = array_merge(maybe_unserialize($order), $downloads);
                        }
                    }

                    $fileIds = $selection_value;
                    $postsIn = array();

                    if (empty($selection_value) || is_null($selection_value)) {
                        foreach ($downloads as $download) {
                            $postsIn[] = $download['file_data']['id'];
                        }
                    } else {
                        // Count order for each file
                        foreach ($fileIds as $fileId) {
                            foreach ($downloads as $download) {
                                if ($download['file_data']['id'] === $fileId) {
                                    $postsIn[] = $fileId;
                                }
                            }
                        }
                    }

                    $postsIn               = array_unique($postsIn);
                    $arguments['post__in'] = $postsIn;
                }

                return $arguments;
            }, 10, 3);
            add_filter('wpfd_statistics_type', function ($type, $selection, $selection_value) {
                if ($selection === 'file_orders') {
                    // phpcs:disable PHPCompatibility.FunctionDeclarations.NewClosure.ClassRefFound -- it's ok ( Closures / anonymous functions could not use "self::" in PHP 5.3 or earlier)
                    return self::STATISTICS_TYPE;
                }

                return $type;
            }, 10, 3);

            // Cloud
            add_filter('wpfda_onedrive_before_save_file_info', array($this, 'wpfdaCloudBeforeSaveFileInfo'));
            add_filter('wpfda_onedrive_business_before_save_file_info', array($this, 'wpfdaCloudBeforeSaveFileInfo'));
            add_filter('wpfda_dropbox_before_save_file_info', array($this, 'wpfdaCloudBeforeSaveFileInfo'));
            add_filter('wpfda_googleDrive_before_save_file_info', array($this, 'wpfdaCloudBeforeSaveFileInfo'));
            add_filter('wpfda_googleDrive_properties', array($this, 'wpfdaGoogleDriveProperties'));

            // Add notification text when product have file
            add_action('woocommerce_product_options_general_product_data', function () {
                // Check product have linked to a file
                global $post;
                $showNotification = get_post_meta($post->ID, '_wpfd_hide_notification', true);

                if ($showNotification !== '' && ($showNotification === false || $showNotification !== 1)) {
                    return;
                }

                $files = get_post_meta($post->ID, '_wpfd_wc_files');
                if ($files !== false && $files !== '' && is_array($files) && count($files)) {
                    wpfd_get_template('html-product-notification.php', array('productId' => $post->ID));
                }
            });
            add_action('wp_ajax_dismiss_wpfd_product_notice', function () {
                $productId = Utilities::getInt('product', 'POST');
                if (!$productId) {
                    wp_send_json_error(array('message' => esc_html__('Product id not found!', 'wpfdAddon')));
                }
                update_post_meta($productId, '_wpfd_hide_notification', 1);
                wp_send_json_success(array('message' => esc_html__('Success!', 'wpfdAddon')));
            });
        }
        // Progress files when order mark as completed
        add_action('woocommerce_after_order_object_save', array($this, 'wpfdaOrderCompleted'), 10, 1);
        // Frontend
        // Load scripts/styles
        add_action('wp_enqueue_scripts', array($this, 'addFrontendScripts'));
        // Add linked products
        add_filter('wpfd_files_info', array($this, 'wpfdaShowLinkedProduct'), 10);
        add_filter('wpfd_file_info', array($this, 'wpfdaShowLinkedProduct'), 10);
        add_filter('wpfda_file_info', array($this, 'wpfdaShowLinkedProductForCloud'), 10, 2);
        // Change button text and link
        add_filter('wpfd_files_info', array($this, 'wpfdaChangeDownload'), 12);
        add_filter('wpfd_file_info', array($this, 'wpfdaChangeDownload'), 12);
        add_filter('wpfda_file_info', array($this, 'wpfdaChangeFileDownload'), 10, 2);
        $this->themeFilters();

        // Check permission when using wpfd download link
        add_action('wpfd_before_download_file', array($this, 'wpfdaCheckPermissionOnWpfdLink'), 10, 2);
        // Filter files when user download category
        add_filter('wpfd_selected_files', function ($files) {
            foreach ($files as $key => $file) {
                if (isset($file->products) && !empty($file->products)) {
                    // phpcs:disable PHPCompatibility.FunctionDeclarations.NewClosure.ClassRefFound -- it's ok ( Closures / anonymous functions could not use "self::" in PHP 5.3 or earlier)
                    if (!isset($file->woo_permission) || (isset($file->woo_permission) && $file->woo_permission === self::ONLY_WOO_PERMISSION)) {
                        unset($files[$key]);
                    }
                }
            }

            return $files;
        });

        add_filter('wpfd_selected_file', function ($file) {
            if (isset($file->products) && !empty($file->products)) {
                // phpcs:disable PHPCompatibility.FunctionDeclarations.NewClosure.ClassRefFound -- it's ok ( Closures / anonymous functions could not use "self::" in PHP 5.3 or earlier)
                if (!isset($file->woo_permission) || (isset($file->woo_permission) && $file->woo_permission === self::ONLY_WOO_PERMISSION)) {
                    return false;
                }
            }

            return $file;
        });

        // Display multiple products
        // phpcs:disable WordPress.Security.NonceVerification.Recommended -- Just for check not use data here
        if (isset($_GET['wpfd_page']) && intval($_GET['wpfd_page']) === 1) {
            add_filter('the_title', function ($title, $id) {
                if (isset($_GET['pids']) && $_GET['pids'] !== '') {
                    // phpcs:enable WordPress.Security.NonceVerification.Recommended
                    $p = get_post($id);
                    // Check is main content area
                    if (in_the_loop() && $p->post_type !== 'product') {
                        return '';
                    }
                }

                return $title;
            }, 10, 2);

            add_filter('the_content', function ($content) {
                // phpcs:disable WordPress.Security.NonceVerification.Recommended -- security below
                if (isset($_GET['pids']) && $_GET['pids'] !== '') {
                    $pids = esc_attr($_GET['pids']);
                    // phpcs:enable WordPress.Security.NonceVerification.Recommended
                    // Security, accept number only
                    $pids = array_map('intval', explode(',', $pids));
                    $pids = implode(',', $pids);

                    return do_shortcode('[products ids="' . $pids . '" columns="3"]');
                }

                return $content;
            });

            add_action('template_redirect', function () {
                if (file_exists(TEMPLATEPATH . '/page.php')) {
                    include(TEMPLATEPATH . '/page.php');
                } elseif (file_exists(TEMPLATEPATH . '/single.php')) {
                    include(TEMPLATEPATH . '/single.php');
                } elseif (file_exists(TEMPLATEPATH . '/singular.php')) {
                    include(TEMPLATEPATH . '/singular.php');
                }
                exit;
            });
        }
        add_filter('woocommerce_customer_available_downloads', function ($downloads, $customer_id) {
            include_once WPFDA_PLUGIN_DIR_PATH . 'app/admin/classes/Woocommerce/WpfdWCDownloadFiles.php';
            $customer_orders = get_posts(array(
                'numberposts' => - 1,
                'meta_key'    => '_customer_user',
                'meta_value'  => get_current_user_id(),
                'post_type'   => wc_get_order_types(),
                'post_status' => 'wc-completed',
            ));
            foreach ($customer_orders as $order) {
                $orderId       = $order->ID;
                $downloadFiles = new WpfdWCDownloadFiles($orderId);
                $files         = $downloadFiles->getDownloads();

                $downloads = array_merge($files, $downloads);
            }

            return $downloads;
        }, 10, 2);

        add_filter('woocommerce_order_get_downloadable_items', function ($downloads, $order) {
            include_once WPFDA_PLUGIN_DIR_PATH . 'app/admin/classes/Woocommerce/WpfdWCDownloadFiles.php';
            $orderId       = $order->get_id();
            $downloadFiles = new WpfdWCDownloadFiles($orderId);
            $files         = $downloadFiles->getDownloads();
            $downloads     = $files + $downloads;

            return $downloads;
        }, 10, 2);

        add_filter('woocommerce_product_file', function ($file, $product, $download_id) {
            $wpfdFiles = $product->get_meta('_wpfd_wc_files');
            if ($wpfdFiles === '') {
                return $file;
            }
            if ('' === $download_id) {
                $file = count($wpfdFiles) ? current($wpfdFiles) : false;
            } elseif (isset($files[$download_id])) {
                $file = $wpfdFiles[$download_id];
            } else {
                $file = false;
            }

            return $file;
        }, 99, 3);

        // Load frontend stylesheet
        add_action('wp_enqueue_scripts', function () {
            wp_enqueue_style('wpfda_frontend', WPFDA_PLUGIN_URL . '/app/site/assets/css/frontend.css');
        });
    }

    /**
     * Check if woocommerce installed
     *
     * @return boolean
     */
    public function isWoo()
    {
        if (defined('WC_VERSION') && version_compare(WC_VERSION, '3.5', 'ge')) {
            return true;
        }

        return false;
    }

    /**
     * Add styles, scripts to admin area
     *
     * @param string $hook Current page
     *
     * @return void
     */
    public function addAdminScripts($hook)
    {
        global $post;
        if (!in_array($hook, array('post.php', 'post-new.php')) || $post->post_type !== 'product') {
            return;
        }

        wp_enqueue_style('wpfd-woo-addon-style', plugins_url('app/admin/assets/css/style.css', WPFDA_PLUGIN_FILE));
        wp_enqueue_script('wpfd-woo-addon-script', plugins_url('app/admin/assets/js/wpfd_woocommerce_addon.js', WPFDA_PLUGIN_FILE), array('jquery'));
        wp_localize_script('wpfd-woo-addon-script', 'wpfdWooAddonVars', array('adminurl' => admin_url()));
    }

    /**
     * Add front-end scripts
     *
     * @param string $hook Current page hook
     *
     * @return void
     */
    public function addFrontendScripts($hook)
    {
        global $post;
        if (!isset($post)) {
            return;
        }
        //if (has_shortcode($post->post_content, 'wpfd_category') || has_shortcode($post->post_content, 'wpfd_file') || has_shortcode($post->post_content, 'wpfd_search')) {
            wp_enqueue_script('wpfd-woo-script', plugins_url('app/site/assets/js/woocommerce.js', WPFDA_PLUGIN_FILE), array('jquery'));
        //}
    }

    /**
     * Add tab to product page
     *
     * @param array $tabs Woocommerce product tabs array
     *
     * @return array
     */
    public function addProductTab($tabs)
    {
        $tabs['wpfd'] = array(
            'label'    => __('WP File Download', 'wpfdAddon'),
            'target'   => 'wpfd_product_data',
            'class'    => array('show_if_downloadable', 'hidden'),
            'priority' => 80,
        );

        return $tabs;
    }

    /**
     * Add wpfd panel to product page
     *
     * @return void
     */
    public function addProductDataPanel()
    {
        wpfd_get_template('html-product-data-wpfd.php');
    }

    /**
     * Include downloads information to files
     *
     * @param array|object|WP_Post $files Post instance(s)
     *
     * @return array|object
     */
    public function wpfdaChangeDownload($files)
    {
        if ($files instanceof WP_Post) {
            return $this->wpfdaChangeFileDownload($files);
        }
        if (!is_array($files)) {
            return $files;
        }
        // Cloud files
        if (is_array($files) && isset($files['ID'])) {
            return $this->wpfdaChangeFileDownload($files);
        }
        $filex = array();
        foreach ($files as $file) {
            $filex[] = $this->wpfdaChangeFileDownload($file);
        }

        return $filex;
    }

    /**
     * Change file download
     *
     * @param object|array $file       File instance
     * @param string       $returnType Type to return
     *
     * @return object
     */
    public function wpfdaChangeFileDownload($file, $returnType = OBJECT)
    {
        $isObject = is_object($file);
        $file = $this->transform($file, OBJECT);
        if (!$isObject) {
            $returnType = ARRAY_A;
        }
        $categoryFrom = apply_filters('wpfdAddonCategoryFrom', $file->catid);
        switch ($categoryFrom) {
            case 'onedrive':
                $config = get_option('_wpfdAddon_onedrive_fileInfo');
                if (!empty($config) && isset($config[$file->catid]) && isset($config[$file->catid][$file->ID])) {
                    $metaData = $config[$file->catid][$file->ID];
                }
                break;
            case 'onedrive_business':
                $config = get_option('_wpfdAddon_onedrive_business_fileInfo');
                if (!empty($config) && isset($config[$file->catid]) && isset($config[$file->catid][$file->ID])) {
                    $metaData = $config[$file->catid][$file->ID];
                }
                break;
            case 'dropbox':
                $config = get_option('_wpfdAddon_dropbox_fileInfo');
                if (!empty($config) && isset($config[$file->catid]) && isset($config[$file->catid][$file->ID])) {
                    $metaData = $config[$file->catid][$file->ID];
                }
                break;
            case 'googleDrive':
                $metaData = array('woo_permission' => isset($file->woo_permission) ? $file->woo_permission : self::ONLY_WOO_PERMISSION);
                break;
            default:
                $metaData = get_post_meta($file->ID, '_wpfd_file_metadata', true);
                break;
        }

        $file->woo_permission = (isset($metaData) && isset($metaData['woo_permission'])) ? esc_attr($metaData['woo_permission']) : self::ONLY_WOO_PERMISSION;

        if (is_object($file) && isset($file->products) && !empty($file->products)) {
            // There is no valid order change linkdownload to add to cart
            $totalProduct = count($file->products);
            if ($totalProduct > 0) {
                $product                = $file->products[0];
                $file->show_add_to_cart = false;

                if ($this->isWooPermission($file) === self::BOTH_PERMISSION) {
                    if (!WpfdHelperFile::checkAccess((array) $file)) {
                        $file->show_add_to_cart = true;
                    }
                } elseif ($this->isWooPermission($file) === self::ONLY_WOO_PERMISSION) {
                    $file->show_add_to_cart = true;
                }

                if ($file->show_add_to_cart === true) {
                    $file->linkdownload = '#';

                    // @var $product WC_Product_Simple
                    $file->viewerlink = $product->is_visible() ? $product->get_permalink() : '';
                    if ($totalProduct > 1) {
                        $pids = array();
                        foreach ($file->products as $prd) {
                            if ($prd) {
                                /* @var $prd WC_Product_Simple */
                                $pids[] = $prd->get_id();
                            }
                        }

                        $file->viewerlink = trailingslashit(site_url()) . '?wpfd_page=1&pids=' . implode(',', $pids);
                    }
                    // Need an mark for check in handlebars template
                    $file->is_product = 1;
                    $file->product_id = $product->get_id();
                }

                $customer_orders = get_posts(array(
                    'numberposts' => - 1,
                    'meta_key'    => '_customer_user',
                    'meta_value'  => get_current_user_id(),
                    'post_type'   => wc_get_order_types(),
                    'post_status' => 'wc-completed',
                ));

                foreach ($customer_orders as $order) {
                    $order         = wc_get_order($order->ID);
                    $items         = $order->get_downloadable_items();
                    $wpfdOrderData = get_post_meta($order->get_id(), '_wpfd_order_data', true);
                    foreach ($items as $item) {
                        if (isset($item['download_id']) && isset($wpfdOrderData[$item['download_id']])) {
                            $orderedFilesId = $wpfdOrderData[$item['download_id']]['file_data']['id'];

                            if ((string) $file->ID === (string) $orderedFilesId) {
                                // Change linkdownload to ordered link file
                                $file->linkdownload         = $item['download_url'];
                                $file->woo_download_granted = 1;

                                return $this->transform($file, $returnType);
                            }
                        }
                    }
                }

                return $this->transform($file, $returnType);
            }
        }

        return $this->transform($file, $returnType);
    }

    /**
     * Transform file instance base on it type
     *
     * @param array|object $object Input object
     * @param string       $type   Output type
     *
     * @return array|stdClass
     */
    private function transform($object, $type)
    {
        if (is_array($object)) {
            if ($type === ARRAY_A) {
                return $object;
            }

            if ($type === OBJECT) {
                $tmpObject = new stdClass;
                foreach ($object as $key => $value) {
                    $tmpObject->{$key} = $value;
                }

                return $tmpObject;
            }
        }

        if (is_object($object)) {
            if ($type === OBJECT) {
                return $object;
            }

            if ($type === ARRAY_A) {
                $tmpObject = array();
                foreach ($object as $key => $value) {
                    $tmpObject[$key] = $value;
                }

                return $tmpObject;
            }
        }

        return $object;
    }

    /**
     * Theme filters
     *
     * @return void
     */
    public function themeFilters()
    {
        add_filter('wpfd_download_data_attributes_handlebars', function ($text) {
            $html = '{{#if woo_download_granted}}';
            $html .= $text;
            $html .= '{{else if is_product}}';
            $html .= 'data-product_id="{{product_id}}"';
            $html .= '{{else}}';
            $html .= $text;
            $html .= '{{/if}}';

            return $html;
        });
        add_filter('wpfd_download_data_attributes', function ($attributes, $currentFile) {
            if (isset($currentFile->is_product) && isset($currentFile->product_id) && intval($currentFile->product_id) > 0) {
                $attributes[] = 'data-product_id="' . $currentFile->product_id . '"';

                return $attributes;
            }

            return $attributes;
        }, 10, 2);
        add_filter('wpfd_download_text_handlebars', function ($text) {
            $html = '{{#if woo_download_granted}}';
            $html .= $text;
            $html .= '{{else if is_product}}';
            $html .= esc_html__('Add to cart', 'wpfdAddon');
            $html .= '{{else}}';
            $html .= $text;
            $html .= '{{/if}}';

            return $html;
        });
        add_filter('wpfd_download_text', function ($text, $currentFile) {
            if (isset($currentFile->woo_download_granted) && $currentFile->woo_download_granted === 1) {
                return $text;
            }
            if (isset($currentFile->is_product) && isset($currentFile->product_id) && intval($currentFile->product_id) > 0) {
                return esc_html__('Add to cart', 'wpfdAddon');
            }

            return $text;
        }, 10, 2);
        // Change icon to shopping-cart-plus
        add_filter('wpfd_download_icon_handlebars', function ($text) {
            $html = '{{#if woo_download_granted}}';
            $html .= $text;
            $html .= '{{else if is_product}}';
            $html .= '<i class="zmdi zmdi-shopping-cart-plus wpfd-add-to-cart"></i>';
            $html .= '{{else}}';
            $html .= $text;
            $html .= '{{/if}}';

            return $html;
        });

        add_filter('wpfd_preview_text_handlebars', function ($text) {
            return '{{#if is_product}}{{ plural productsCount \'' . esc_html__('View Product(s)', 'wpfdAddon') . '\' }}{{else}}' . $text . '{{/if}}';
        });
        add_filter('wpfd_preview_text', function ($text, $currentFile) {
            if (isset($currentFile->is_product) && isset($currentFile->product_id) && intval($currentFile->product_id) > 0) {
                return _n('View Product', 'View Products', count($currentFile->products), 'wpfdAddon');
            }

            return $text;
        }, 10, 2);
        // Change icon to shopping-cart-plus
        add_filter('wpfd_download_icon', function ($html, $currentFile) {
            if (!isset($currentFile->woo_download_granted) && isset($currentFile->is_product) && isset($currentFile->product_id) && intval($currentFile->product_id) > 0) {
                return '<i class="zmdi zmdi-shopping-cart-plus wpfd-add-to-cart"></i>';
            }

            return $html;
        }, 10, 2);
        add_filter('wpfd_preview_classes_handlebars', function ($classes) {
            return str_replace('wpfdlightbox', '{{#unless is_product}}wpfdlightbox{{/unless}}', $classes);
        });
        add_filter('wpfd_preview_classes', function ($classes, $file) {
            if (isset($file->is_product) && $file->is_product) {
                return str_replace('wpfdlightbox', '', $classes);
            }

            return $classes;
        }, 10, 2);
    }

    /**
     * Check permission
     *
     * @param object $file File instance
     *
     * @return boolean|string True if use WooCommerce permission only
     */
    public function isWooPermission($file)
    {
        if (is_object($file) && isset($file->woo_permission)) {
            if (in_array($file->woo_permission, array(self::ONLY_WOO_PERMISSION, self::BOTH_PERMISSION))) {
                return $file->woo_permission;
            }
        }

        // Default value
        if (isset($file->products) && !empty($file->products)) {
            return self::ONLY_WOO_PERMISSION;
        }

        return false;
    }

    /**
     * Check file has products
     *
     * @param boolean $return True or false
     * @param WP_Post $file   File object
     *
     * @return boolean
     */
    public function wpfdaFileHasProducts($return, $file)
    {
        if (isset($file->products) && !empty($file->products)) {
            return true;
        }

        return false;
    }

    /**
     * File row data
     *
     * @param array  $data Data
     * @param object $file File data
     *
     * @return array
     */
    public function wpfdaFileRowData($data, $file)
    {
        $productIds = array();
        if (isset($file->products) && is_array($file->products)) {
            foreach ($file->products as $product) {
                if ($product) {
                    $productIds[] = $product->get_id();
                }
            }
            if (count($productIds) > 0) {
                $data['products'] = implode(',', $productIds);
            }
        }

        return $data;
    }

    /**
     * Assigin linked products to file object
     *
     * @param WP_Post $files File data
     *
     * @return mixed
     */
    public function wpfdaShowLinkedProduct($files)
    {
        if (!$files || empty($files)) {
            return $files;
        }

        // Check if file is object
        if (is_object($files) && isset($files->ID)) {
            $productIds      = $this->getLinkedProduct($files->ID);
            $files->products = array();
            if (!empty($productIds)) {
                foreach ($productIds as $productId) {
                    $product = wc_get_product($productId);
                    if (!$product) {
                        continue;
                    }
                    $files->products[] = $product;
                }
            }
            $files->productsCount = count($files->products);

            return $files;
        }

        // Check is single file
        if (is_array($files) && isset($files['ID']) && is_numeric($files['ID'])) {
            $productIds        = $this->getLinkedProduct($files['ID']);
            $files['products'] = array();
            if (!empty($productIds)) {
                foreach ($productIds as $productId) {
                    $product = wc_get_product($productId);
                    if (!$product) {
                        continue;
                    }
                    $files['products'][] = $product;
                }
            }
            $files['productsCount'] = count($files['products']);

            return $files;
        }

        foreach ($files as $file) {
            $productIds     = $this->getLinkedProduct($file->ID);
            $file->products = array();
            if (!empty($productIds)) {
                foreach ($productIds as $productId) {
                    $product = wc_get_product($productId);
                    if (!$product) {
                        continue;
                    }
                    $file->products[] = $product;
                }
            }
            $file->productsCount = count($file->products);
        }

        return $files;
    }

    /**
     * Show linked products for cloud
     *
     * @param array|object $data File data
     * @param string       $type Output type
     *
     * @return array|stdClass
     */
    public function wpfdaShowLinkedProductForCloud($data, $type)
    {
        $isObject = is_object($data);

        $data = $this->transform($data, ARRAY_A);
        if (in_array($type, wpfd_get_support_cloud())) {
            switch ($type) {
                case 'onedrive':
                    $config             = get_option('_wpfdAddon_onedrive_fileInfo');
                    $linkedProductsData = get_option('_wpfdAddon_onedrive_products_linked');

                    if (!empty($linkedProductsData) && isset($linkedProductsData[$data['catid']]) && isset($linkedProductsData[$data['catid']][$data['id']])) {
                        $productIds       = isset($linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked']) ?
                            $linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked'] : '';
                        $data['products'] = array();
                        if (!empty($productIds)) {
                            foreach ($productIds as $productId) {
                                $data['products'][] = wc_get_product($productId);
                            }
                        }
                        $data['productsCount'] = count($data['products']);
                    }

                    if (!empty($config) && isset($config[$data['catid']]) && isset($config[$data['catid']][$data['id']])) {
                        $data['woo_permission'] = isset($config[$data['catid']][$data['id']]['woo_permission']) ?
                            $config[$data['catid']][$data['id']]['woo_permission'] : self::ONLY_WOO_PERMISSION;
                    }

                    break;
                case 'onedrive_business':
                    $config             = get_option('_wpfdAddon_onedrive_business_fileInfo');
                    $linkedProductsData = get_option('_wpfdAddon_onedrive_business_products_linked');

                    if (!empty($linkedProductsData) && isset($linkedProductsData[$data['catid']]) && isset($linkedProductsData[$data['catid']][$data['id']])) {
                        $productIds       = isset($linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked']) ?
                            $linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked'] : '';
                        $data['products'] = array();
                        if (!empty($productIds)) {
                            foreach ($productIds as $productId) {
                                $data['products'][] = wc_get_product($productId);
                            }
                        }
                        $data['productsCount'] = count($data['products']);
                    }

                    if (!empty($config) && isset($config[$data['catid']]) && isset($config[$data['catid']][$data['id']])) {
                        $data['woo_permission'] = isset($config[$data['catid']][$data['id']]['woo_permission']) ?
                            $config[$data['catid']][$data['id']]['woo_permission'] : self::ONLY_WOO_PERMISSION;
                    }

                    break;
                case 'dropbox':
                    $config             = get_option('_wpfdAddon_dropbox_fileInfo');
                    $linkedProductsData = get_option('_wpfdAddon_dropbox_products_linked');

                    if (!empty($linkedProductsData) && isset($linkedProductsData[$data['catid']]) && isset($linkedProductsData[$data['catid']][$data['id']])) {
                        $productIds       = isset($linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked']) ?
                            $linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked'] : '';
                        $data['products'] = array();
                        if (!empty($productIds)) {
                            foreach ($productIds as $productId) {
                                $data['products'][] = wc_get_product($productId);
                            }
                        }
                        $data['productsCount'] = count($data['products']);
                    }

                    if (!empty($config) && isset($config[$data['catid']]) && isset($config[$data['catid']][$data['id']])) {
                        $data['woo_permission'] = isset($config[$data['catid']][$data['id']]['woo_permission']) ?
                            $config[$data['catid']][$data['id']]['woo_permission'] : self::ONLY_WOO_PERMISSION;
                    }
                    break;
                case 'googleDrive':
                    $linkedProductsData = get_option('_wpfdAddon_googleDrive_products_linked');

                    if (!empty($linkedProductsData) && isset($linkedProductsData[$data['catid']]) && isset($linkedProductsData[$data['catid']][$data['id']])) {
                        $productIds       = isset($linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked']) ?
                            $linkedProductsData[$data['catid']][$data['id']]['_wpfd_products_linked'] : '';
                        $data['products'] = array();
                        if (!empty($productIds)) {
                            foreach ($productIds as $productId) {
                                $data['products'][] = wc_get_product($productId);
                            }
                        }
                        $data['productsCount'] = count($data['products']);
                    }
                    break;
                default:
                    break;
            }
        }
        if ($isObject) {
            $data = $this->transform($data, OBJECT);
        }

        return $data;
    }

    /**
     * Filter data on save onedrive file
     *
     * @param array $fileInfo File information
     *
     * @return array
     */
    public function wpfdaCloudBeforeSaveFileInfo($fileInfo)
    {
        $permission = Utilities::getInput('woo_permission', 'POST', 'string');
        if (!is_null($permission)) {
            $fileInfo['woo_permission'] = $permission;
        }

        return $fileInfo;
    }

    /**
     * Merge Google Drive properties
     *
     * @param array $properties Properties
     *
     * @return array
     */
    public function wpfdaGoogleDriveProperties($properties)
    {
        return array_merge(array('woo_permission'), $properties);
    }

    /**
     * Add wpfd files on product save
     *
     * @param integer $productId Product id
     *
     * @return void
     */
    public function wpfdaWooAddFilesSave($productId)
    {
        // Text Field
        $_wpfd_wc_files_name   = Utilities::getInput('_wpfd_wc_files_name', 'POST', 'none');
        $_wpfd_wc_files_id     = Utilities::getInput('_wpfd_wc_files_id', 'POST', 'none');
        $_wpfd_wc_files_catid  = Utilities::getInput('_wpfd_wc_files_catid', 'POST', 'none');
        $_wpfd_download_limit  = Utilities::getInput('_wpfd_download_limit', 'POST', 'string');
        $_wpfd_download_expiry = Utilities::getInput('_wpfd_download_expiry', 'POST', 'string');
        if (!empty($_wpfd_wc_files_name) && !empty($_wpfd_wc_files_id) && !empty($_wpfd_wc_files_catid)) {
            $files = $this->wpfdaPrepareDownloads($_wpfd_wc_files_name, $_wpfd_wc_files_id, $_wpfd_wc_files_catid);

            // Unlink removed files from product
            $this->wpfdaUnlinkRemovedProductFromFile($_wpfd_wc_files_id, $productId);
            // Connect file to product
            update_post_meta($productId, '_wpfd_wc_files', $files);
            foreach ($files as $file) {
                $this->wpfdaLinkProductToFile($file['id'], $file['catid'], $productId);
            }
            update_post_meta($productId, '_wpfd_download_limit', $_wpfd_download_limit);
            update_post_meta($productId, '_wpfd_download_expiry', $_wpfd_download_expiry);
        } else { // Empty files
            $this->wpfdaUnlinkAllFiles($productId);
            delete_post_meta($productId, '_wpfd_wc_files');
            delete_post_meta($productId, '_wpfd_download_limit');
            delete_post_meta($productId, '_wpfd_download_expiry');
        }
    }

    /**
     * Unlink all files from a product
     *
     * @param integer $productId Product id
     *
     * @return void
     */
    private function wpfdaUnlinkAllFiles($productId)
    {
        $files = $this->getLinkedFiles($productId);
        if ($files === '' || !is_array($files)) {
            return;
        }
        foreach ($files as $file) {
            $this->wpfdaUnlinkProductFromFile($file['id'], $productId);
        }
    }

    /**
     * Unlink removed files
     *
     * @param array   $fileIds   File ids
     * @param integer $productId Product id
     *
     * @return void
     */
    private function wpfdaUnlinkRemovedProductFromFile($fileIds, $productId)
    {
        $fileIds = array_map('intval', $fileIds);
        $files   = $this->getLinkedFiles($productId);

        if ($files === '' || !is_array($files)) {
            return;
        }

        foreach ($files as $file) {
            if (!in_array($fileIds, $file['id'])) {
                $this->wpfdaUnlinkProductFromFile($file['id'], $productId);
            }
        }
    }

    /**
     * Unlink product from file
     *
     * @param array   $fileId    File id
     * @param integer $productId Product id
     *
     * @return void
     */
    private function wpfdaUnlinkProductFromFile($fileId, $productId)
    {
        $linkedProducts = get_post_meta($fileId, '_wpfd_products_linked', true);
        if (is_array($linkedProducts) && count($linkedProducts)) {
            foreach ($linkedProducts as $key => $linkedProductId) {
                if (intval($linkedProductId) === intval($productId)) {
                    unset($linkedProducts[$key]);
                }
            }
            if (empty($linkedProducts)) {
                delete_post_meta($fileId, '_wpfd_products_linked');
            } else {
                update_post_meta($fileId, '_wpfd_products_linked', $linkedProducts);
            }
        }
    }

    /**
     * Link product to file
     *
     * @param string|integer $fileId    File id
     * @param integer        $catId     Category Id
     * @param integer        $productId Product id
     *
     * @return void
     */
    private function wpfdaLinkProductToFile($fileId, $catId, $productId)
    {
        $categoryFrom = apply_filters('wpfdAddonCategoryFrom', $catId);
        switch ($categoryFrom) {
            case 'onedrive':
                $this->linkProductToOnedriveFile($fileId, $catId, $productId);
                break;
            case 'onedrive_business':
                $this->linkProductToOnedriveBusinessFile($fileId, $catId, $productId);
                break;
            case 'dropbox':
                $this->linkProductToDropboxFile($fileId, $catId, $productId);
                break;
            case 'googleDrive':
                $this->linkProductToGoogleDriveFile($fileId, $catId, $productId);
                break;
            default:
                $this->wpfdLinkProductToLocalFile($fileId, $productId);
                break;
        }
    }

    /**
     * Link product to Onedrive file
     *
     * @param string|integer $fileId    File id
     * @param integer        $catId     Category Id
     * @param integer        $productId Product id
     *
     * @return void
     */
    private function linkProductToOnedriveFile($fileId, $catId, $productId)
    {
        $onedriveInfos = get_option('_wpfdAddon_onedrive_products_linked');

        if (false === $onedriveInfos) {
            $onedriveInfos = array(
                $catId => array(
                    $fileId => array(
                        '_wpfd_products_linked' => array($productId)
                    )
                )
            );
        } else {
            $linkedProducts                                          = isset($onedriveInfos[$catId][$fileId]['_wpfd_products_linked']) ? $onedriveInfos[$catId][$fileId]['_wpfd_products_linked'] : array();
            $data                                                    = array_unique(array_merge($linkedProducts, array($productId)));
            $onedriveInfos[$catId][$fileId]['_wpfd_products_linked'] = $data;
        }

        update_option('_wpfdAddon_onedrive_products_linked', $onedriveInfos);
    }
    /**
     * Link product to Onedrive Business file
     *
     * @param string|integer $fileId    File id
     * @param integer        $catId     Category Id
     * @param integer        $productId Product id
     *
     * @return void
     */
    private function linkProductToOnedriveBusinessFile($fileId, $catId, $productId)
    {
        $onedriveInfos = get_option('_wpfdAddon_onedrive_business_products_linked');

        if (false === $onedriveInfos) {
            $onedriveInfos = array(
                $catId => array(
                    $fileId => array(
                        '_wpfd_products_linked' => array($productId)
                    )
                )
            );
        } else {
            $linkedProducts                                          = isset($onedriveInfos[$catId][$fileId]['_wpfd_products_linked']) ? $onedriveInfos[$catId][$fileId]['_wpfd_products_linked'] : array();
            $data                                                    = array_unique(array_merge($linkedProducts, array($productId)));
            $onedriveInfos[$catId][$fileId]['_wpfd_products_linked'] = $data;
        }

        update_option('_wpfdAddon_onedrive_business_products_linked', $onedriveInfos);
    }
    /**
     * Link product to Dropbox file
     *
     * @param string|integer $fileId    File id
     * @param integer        $catId     Category Id
     * @param integer        $productId Product id
     *
     * @return void
     */
    private function linkProductToDropboxFile($fileId, $catId, $productId)
    {
        $dropboxInfos = get_option('_wpfdAddon_dropbox_products_linked');

        if (false === $dropboxInfos) {
            $dropboxInfos = array(
                $catId => array(
                    $fileId => array(
                        '_wpfd_products_linked' => array($productId)
                    )
                )
            );
        } else {
            $linkedProducts                                         = isset($dropboxInfos[$catId][$fileId]['_wpfd_products_linked']) ? $dropboxInfos[$catId][$fileId]['_wpfd_products_linked'] : array();
            $data                                                   = array_unique(array_merge($linkedProducts, array($productId)));
            $dropboxInfos[$catId][$fileId]['_wpfd_products_linked'] = $data;
        }

        update_option('_wpfdAddon_dropbox_products_linked', $dropboxInfos);
    }

    /**
     * Link product to Google Drive file
     *
     * @param string  $fileId    File id
     * @param integer $catId     Category id
     * @param integer $productId Product id
     *
     * @return void
     */
    private function linkProductToGoogleDriveFile($fileId, $catId, $productId)
    {
        $googleDriveInfos = get_option('_wpfdAddon_googleDrive_products_linked');

        if (false === $googleDriveInfos) {
            $googleDriveInfos = array(
                $catId => array(
                    $fileId => array(
                        '_wpfd_products_linked' => array($productId)
                    )
                )
            );
        } else {
            $linkedProducts                                             = isset($googleDriveInfos[$catId][$fileId]['_wpfd_products_linked']) ? $googleDriveInfos[$catId][$fileId]['_wpfd_products_linked'] : array();
            $data                                                       = array_unique(array_merge($linkedProducts, array($productId)));
            $googleDriveInfos[$catId][$fileId]['_wpfd_products_linked'] = $data;
        }

        update_option('_wpfdAddon_googleDrive_products_linked', $googleDriveInfos);
    }

    /**
     * Link product to local file
     *
     * @param integer $fileId    File id
     * @param string  $productId Product id
     *
     * @return void
     */
    private function wpfdLinkProductToLocalFile($fileId, $productId)
    {
        $linkedProducts = get_post_meta($fileId, '_wpfd_products_linked', true);
        if ($linkedProducts === '') {
            $linkedProducts = array();
        }

        update_post_meta($fileId, '_wpfd_products_linked', array_unique(array_merge($linkedProducts, array($productId))));
    }

    /**
     * Prepare download file array to store
     *
     * @param array $file_names  List file name
     * @param array $file_ids    List file id
     * @param array $file_catids List file category id
     *
     * @return array
     */
    private function wpfdaPrepareDownloads($file_names, $file_ids, $file_catids)
    {
        $downloads = array();

        if (!empty($file_ids)) {
            $file_ids_size = count($file_ids);

            for ($i = 0; $i < $file_ids_size; $i ++) {
                if (isset($file_ids[$i])) {
                    $downloads[] = array(
                        'name'  => wc_clean($file_names[$i]),
                        'id'    => wc_clean($file_ids[$i]),
                        'catid' => wc_clean($file_catids[$i]),
                    );
                }
            }
        }

        return $downloads;
    }

    /**
     * Get linked file to product
     *
     * @param integer $productId Product id
     *
     * @return array
     */
    private function getLinkedFiles($productId)
    {
        return get_post_meta($productId, '_wpfd_wc_files', true);
    }

    /**
     * Get linked product of file
     *
     * @param integer $fileId File id
     *
     * @return array
     */
    private function getLinkedProduct($fileId)
    {
        return get_post_meta($fileId, '_wpfd_products_linked', true);
    }

    /**
     * Grant download access on order completed
     *
     * @param WC_Order $order WC_Order object
     *
     * @return void
     */
    public function wpfdaOrderCompleted($order)
    {
        if ($order->get_status() !== 'completed') {
            return;
        }
        // Create meta data store current order download status
        $items        = $order->get_items();
        $downloadData = get_post_meta($order->get_id(), '_wpfd_order_data', true);
        if ($downloadData === '') {
            $downloadData = array();
        }

        foreach ($items as $item) {
            // Get downloadable files
            $product = $item->get_product();
            $files   = $this->getLinkedFiles($product->get_id());
            if (is_array($files) && count($files)) {
                // Get max download
                $download_limit   = $product->get_meta('_wpfd_download_limit', true);
                $download_expires = $product->get_meta('_wpfd_download_expiry', true);
                // Calculate expiries date
                if ($download_expires > 0) {
                    $from_date      = $order->get_date_completed() ? $order->get_date_completed()->format('Y-m-d') : current_time('mysql', true);
                    $access_expires = strtotime($from_date . ' + ' . $download_expires . ' DAY');
                }
                $downloads_remaining = 0 > $download_limit ? '' : ($download_limit === '' ? $download_limit : $download_limit * $item->get_quantity());
                foreach ($files as $file) {
                    // Check file already granted
                    $break = false;
                    if (is_array($downloadData) && count($downloadData)) {
                        foreach ($downloadData as $download) {
                            if ($download['product_id'] === $item->get_product_id() && $download['file_data']['id'] === $file['id']) {
                                $break = true;
                            }
                        }
                    }
                    if ($break) {
                        continue;
                    }
                    // todo: get data object with real path...should we need this?
                    // $fileData = $this->getFile($file['id']);
                    $download_id                = wp_generate_uuid4();
                    $downloadData[$download_id] = array(
                        'download_id'         => $download_id,
                        'product_id'          => $item->get_product_id(),
                        'order_key'           => $order->get_order_key('edit'),
                        'file_data'           => $file,
                        // array('id','catid','name')
                        'user_id'             => $order->get_customer_id('edit'),
                        'access_granted'      => current_time('timestamp', true),
                        'access_expires'      => isset($access_expires) ? $access_expires : '',
                        'downloads_remaining' => isset($downloads_remaining) ? $downloads_remaining : '',
                        // todo: Add download_remaining support
                        'download_count'      => 0,
                    );

                    // Add order statistics
                    /* @var $fileModel WpfdModelFile */
                    WpfdHelperFile::addStatisticsRow($file['id'], self::STATISTICS_TYPE);
                }
            }
        }
        update_post_meta($order->get_id(), '_wpfd_order_data', $downloadData);
    }

    /**
     * Add woocommerce permission field
     *
     * @param array $data File data
     *
     * @return void
     */
    public function wpfdaAddWocommercePermissionField($data)
    {
        if (isset($data['products']) && !empty($data['products'])) {
            $value = isset($data['woo_permission']) ? esc_html($data['woo_permission']) : self::ONLY_WOO_PERMISSION;

            wpfd_get_template('html-file-permission-field.php', array('value' => $value));
        }
    }

    /**
     * Add woocommerce permission field
     *
     * @param array $metadata Meta data array
     * @param array $data     Field datas
     *
     * @return array
     */
    public function wpfdaSaveWooPermissionField($metadata, $data)
    {
        $permission                 = Utilities::getInput('woo_permission', 'POST', 'string');
        $metadata['woo_permission'] = !is_null($permission) ? esc_html($permission) : self::ONLY_WOO_PERMISSION;

        return $metadata;
    }

    /**
     * Check permission on wpfd download link
     *
     * @param object $file File object
     * @param array  $meta File metadata
     *
     * @return void
     */
    public function wpfdaCheckPermissionOnWpfdLink($file, $meta)
    {
        if (isset($file->products) && !empty($file->products)) {
            if (!isset($file->woo_permission) || (isset($file->woo_permission) && $file->woo_permission === self::ONLY_WOO_PERMISSION)) {
                wpfd_get_template('html-access-denied.php');
                exit();
            }
        }
    }

    /**
     * Locate addon template
     *
     * @param string $template      Template path
     * @param string $template_name Template name
     *
     * @return string
     */
    public function wpfdaLocateTemplate($template, $template_name)
    {
        if (file_exists($template)) {
            return $template;
        }
        $wooAddonTemplatePath = WPFDA_PLUGIN_DIR_PATH . 'app/admin/classes/Woocommerce/templates/';
        $templates_name       = scandir($wooAddonTemplatePath);

        if (in_array($template_name, $templates_name)) {
            return $wooAddonTemplatePath . $template_name;
        }
    }
}
