<?php
/**
 * WP File Download
 *
 * @package WP File Download
 * @author  Joomunited
 * @version 1.0
 */

use Joomunited\WPFramework\v1_0_5\Utilities;
use Joomunited\WPFramework\v1_0_5\Controller;
use Joomunited\WPFramework\v1_0_5\Model;
use Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Filesystem;

/**
 * Class WpfdAddonControllerCategory
 */
class WpfdAddonControllerCategory extends Controller
{
    /**
     * Google Drive category instance
     *
     * @var boolean
     */
    protected $cloudCategory;
    /**
     * Dropbox category instance
     *
     * @var boolean
     */
    protected $dropboxCategory;
    /**
     * Onedrive category instance
     *
     * @var boolean
     */
    protected $onedriveCategory;

    /**
     * Onedrive Business category instance
     *
     * @var boolean
     */
    protected $onedriveBusinessCategory;

    /**
     * WpfdAddonControllerCategory constructor.
     */
    public function __construct()
    {
        Application::getInstance('WpfdAddon');
        $this->cloudCategory = Model::getInstance('cloudcategory');
        $this->dropboxCategory = Model::getInstance('dropboxcategory');
        Application::getInstance('WpfdAddon');
        $this->onedriveCategory = Model::getInstance('onedrivecategory');
        Application::getInstance('WpfdAddon');
        $this->onedriveBusinessCategory = Model::getInstance('onedrivebusinesscategory');
    }

    /**
     * Add category
     *
     * @return void
     */
    public function addCategory()
    {
        $categoryName =  Utilities::getInput('name', 'POST', 'string');
        if (is_null($categoryName) || empty($categoryName)) {
            $categoryName = esc_html__('New Google Drive', 'wpfdAddon');
        }
        $parentId = Utilities::getInput('parentId', 'POST', 'int');
        if (is_null($parentId)) {
            $parentId = 0;
        }
        $categoryName = $this->getNewCategoryName($categoryName, $parentId);
        $id = $this->cloudCategory->addCategory($categoryName, $parentId, $this->getNewCategoryPosition());
        if ($id) {
            $user_id = get_current_user_id();
            if ($user_id) {
                $user_categories = get_user_meta($user_id, 'wpfd_user_categories', true);
                if (is_array($user_categories)) {
                    if (!in_array($id, $user_categories)) {
                        $user_categories[] = $id;
                    }
                } else {
                    $user_categories = array($id);
                }
                update_user_meta($user_id, 'wpfd_user_categories', $user_categories);
            }
            $this->exitStatus(true, array('id_category' => $id, 'name' => $categoryName));
        }
        $this->exitStatus('error while adding category'); //todo: translate
    }


    /**
     * Delete category
     *
     * @return void
     */
    public function deleteCategory()
    {
        $category = Utilities::getInt('id_category');
        $children = $this->cloudCategory->getChildren($category);
        if ($this->cloudCategory->deleteCategory($category)) {
            $children[] = $category;
            foreach ($children as $child) {
                $dir = WpfdBase::getFilesPath($child);
                Filesystem::rmdir($dir);
                if ((int) $child === $category) {
                    continue;
                }
                $this->cloudCategory->deleteCategory($child);
            }
            $this->exitStatus(true);
        }
        $this->exitStatus('error while deletting category');
    }

    /**
     * Create drop category
     *
     * @return void
     */
    public function addDropCategory()
    {
        $categoryName =  Utilities::getInput('name', 'POST', 'string');
        if (is_null($categoryName) || empty($categoryName)) {
            $categoryName = esc_html__('New Dropbox', 'wpfdAddon');
        }
        $parentId = Utilities::getInput('parentId', 'POST', 'int');
        if (is_null($parentId)) {
            $parentId = 0;
        }
        $categoryName = $this->getNewCategoryName($categoryName, $parentId);
        $id = $this->dropboxCategory->addDropCategory($categoryName, $parentId, $this->getNewCategoryPosition());
        if ($id) {
            $user_id = get_current_user_id();
            if ($user_id) {
                $user_categories = get_user_meta($user_id, 'wpfd_user_categories', true);
                if (is_array($user_categories)) {
                    if (!in_array($id, $user_categories)) {
                        $user_categories[] = $id;
                    }
                } else {
                    $user_categories = array($id);
                }
                update_user_meta($user_id, 'wpfd_user_categories', $user_categories);
            }
            $this->exitStatus(true, array('id_category' => $id, 'name' => $categoryName));
        }
        $this->exitStatus('error while adding dropbox'); //todo: translate
    }

    /**
     * Delete dropbox category
     *
     * @return   void
     * @internal param dropbox $delete
     * @internal param array $datas
     */
    public function deleteDropboxCategory()
    {
        $category = Utilities::getInt('id_category');
        $this->dropboxCategory->deleteDropboxCategory($category);
        $this->exitStatus(true);
    }

    /**
     * Retun status
     *
     * @param mixed $status Status to response
     * @param array $datas  Data include to response
     *
     * @return void
     */
    protected function exitStatus($status = '', $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        die();
    }

    /**
     * Create onedrive category
     *
     * @return void
     */
    public function addOneDriveCategory()
    {
        $categoryName =  Utilities::getInput('name', 'POST', 'string');
        if (is_null($categoryName) || empty($categoryName)) {
            $categoryName = esc_html__('New OneDrive', 'wpfdAddon');
        }
        $parentId = Utilities::getInput('parentId', 'POST', 'int');
        if (is_null($parentId)) {
            $parentId = 0;
        }
        $categoryName = $this->getNewCategoryName($categoryName, $parentId);
        $return = $this->onedriveCategory->addOneDriveCategory($categoryName, $parentId, $this->getNewCategoryPosition());

        if (is_array($return)) {
            if ($return['success']) {
                $user_id = get_current_user_id();
                if ($user_id) {
                    $user_categories = get_user_meta($user_id, 'wpfd_user_categories', true);
                    if (is_array($user_categories)) {
                        if (!in_array($return['id'], $user_categories)) {
                            $user_categories[] = $return['id'];
                        }
                    } else {
                        $user_categories = array($return['id']);
                    }
                    update_user_meta($user_id, 'wpfd_user_categories', $user_categories);
                }
                $this->exitStatus(true, array(
                    'id_category' => $return['id'],
                    'name'        => $categoryName
                ));
            } else {
                $this->exitStatus($return['message']);
            }
        }

        $this->exitStatus('error while adding OneDrive'); //todo: translate
    }

    /**
     * Delete onedrive category
     *
     * @return   void
     * @internal param OneDrive $delete
     * @internal param array $datas
     */
    public function deleteOneDriveCategory()
    {
        $category = Utilities::getInt('id_category');
        $children = $this->onedriveCategory->getChildren($category);
        if ($this->onedriveCategory->deleteOneDriveCategory($category)) {
            $children[] = $category;
            foreach ($children as $child) {
                $dir = WpfdBase::getFilesPath($child);
                Filesystem::rmdir($dir);
                if ((int) $child === $category) {
                    continue;
                }
                $this->onedriveCategory->deleteOneDriveCategory($child);
            }
            $this->exitStatus(true);
        }
        $this->exitStatus('error while deletting category');
    }

    /**
     * Create onedrive business category
     *
     * @return void
     */
    public function addOneDriveBusinessCategory()
    {
        $categoryName =  Utilities::getInput('name', 'POST', 'string');
        if (is_null($categoryName) || empty($categoryName)) {
            $categoryName = esc_html__('New OneDrive Business', 'wpfdAddon');
        }
        $parentId = Utilities::getInput('parentId', 'POST', 'int');
        if (is_null($parentId)) {
            $parentId = 0;
        }
        $categoryName = $this->getNewCategoryName($categoryName, $parentId);
        $return = $this->onedriveBusinessCategory->addCategory($categoryName, $parentId, $this->getNewCategoryPosition());

        if (is_array($return)) {
            if ($return['success']) {
                $user_id = get_current_user_id();
                if ($user_id) {
                    $user_categories = get_user_meta($user_id, 'wpfd_user_categories', true);
                    if (is_array($user_categories)) {
                        if (!in_array($return['id'], $user_categories)) {
                            $user_categories[] = $return['id'];
                        }
                    } else {
                        $user_categories = array($return['id']);
                    }
                    update_user_meta($user_id, 'wpfd_user_categories', $user_categories);
                }
                $this->exitStatus(true, array(
                    'id_category' => $return['id'],
                    'name'        => $categoryName
                ));
            } else {
                $this->exitStatus($return['message']);
            }
        }

        $this->exitStatus('error while adding OneDrive'); //todo: translate
    }

    /**
     * Delete onedrive Business category
     *
     * @return   void
     * @internal param OneDrive $delete
     * @internal param array $datas
     */
    public function deleteOneDriveBusinessCategory()
    {
        $category = Utilities::getInt('id_category');
        $children = $this->onedriveBusinessCategory->getChildren($category);
        if ($this->onedriveBusinessCategory->deleteOneDriveBusinessCategory($category)) {
            $children[] = $category;
            foreach ($children as $child) {
                $dir = WpfdBase::getFilesPath($child);
                Filesystem::rmdir($dir);
                if ((int) $child === $category) {
                    continue;
                }
                $this->onedriveBusinessCategory->deleteOneDriveCategory($child);
            }
            $this->exitStatus(true);
        }
        $this->exitStatus('error while deleting category');
    }

    /**
     * Update category title
     *
     * @param integer $category Term id
     * @param string  $title    Title
     *
     * @return boolean
     */
    public function updateTitle($category, $title)
    {
        $result = wp_update_term($category, 'wpfd-category', array(
            'name' => $title,
            'slug' => sanitize_title($title),
        ));
        if (is_wp_error($result)) { //try again with other slug
            $result = wp_update_term($category, 'wpfd-category', array(
                'name' => $title,
                'slug' => sanitize_title($title) . '-' . time(),
            ));
        }
        if (is_wp_error($result)) {
            return false;
        }

        return true;
    }

    /**
     * Check new category name and span it with number
     *
     * @param string  $categoryName New category name
     * @param integer $parentId     Parent Id
     *
     * @return string
     */
    public function getNewCategoryName($categoryName, $parentId = 0)
    {
        // Check term exists
        $termSpan = 0;
        $checkTitle = $categoryName;
        if (function_exists('term_exists')) {
            while (is_array(term_exists($checkTitle, 'wpfd-category', $parentId))) {
                $termSpan++;
                $checkTitle = $categoryName . ' ' . (string) $termSpan;
            }
        }
        if ($termSpan > 0) {
            $categoryName .= ' ' . (string) $termSpan;
        }

        return $categoryName;
    }

    /**
     * Get new category position option
     *
     * @return string
     */
    protected function getNewCategoryPosition()
    {
        Application::getInstance('Wpfd');
        $configModel = Model::getInstance('config');
        if (!$configModel) {
            return 'end';
        }

        $config = $configModel->getConfig();

        if (!isset($config['new_category_position'])) {
            return 'end';
        }

        return $config['new_category_position'];
    }
}
