<?php
/**
 * WP File Download
 *
 * @package WP File Download
 * @author  Joomunited
 * @version 1.0
 */

use \Joomunited\WPFramework\v1_0_5\Controller;
use \Joomunited\WPFramework\v1_0_5\Form;
use \Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Model;

defined('ABSPATH') || die();

/**
 * Class WpfdAddonControllerCloud
 */
class WpfdAddonControllerDropbox extends Controller
{

    /**
     * Dropbox category instance
     *
     * @var boolean
     */
    protected $dropboxCategory;


    /**
     * WpfdAddonControllerDropbox constructor.
     */
    public function __construct()
    {
        $app             = Application::getInstance('WpfdAddon');
        $path_wpfdhelper = $app->getPath() . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'helpers';
        $path_wpfdhelper .= DIRECTORY_SEPARATOR . 'WpfdHelper.php';
        require_once $path_wpfdhelper;

        $path_wpfdaddondropbox = $app->getPath() . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'classes';
        $path_wpfdaddondropbox .= DIRECTORY_SEPARATOR . 'WpfdAddonDropbox.php';
        require_once $path_wpfdaddondropbox;

        Application::getInstance('WpfdAddon');
        $this->dropboxCategory = Model::getInstance('dropboxcategory');
    }

    /**
     * Update term order
     *
     * @param integer $term_id Term id
     * @param integer $order   Order number
     *
     * @return void
     */
    public function updateTermOrder($term_id, $order)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                'UPDATE ' . $wpdb->terms . ' SET term_group = %d WHERE term_id = %d',
                (int) $order,
                (int) $term_id
            )
        );
    }

    /**
     * Save dropbox Params
     *
     * @return void
     */
    public function saveDropboxParams()
    {
        $form     = new Form();
        $formfile = Application::getInstance('WpfdAddon')->getPath() . DIRECTORY_SEPARATOR . 'admin';
        $formfile .= DIRECTORY_SEPARATOR . 'forms' . DIRECTORY_SEPARATOR . 'dropbox_config.xml';

        if (!$form->load($formfile)) {
            $this->redirect('admin.php?page=wpfd-config&error=1');
        }

        if (!$form->validate()) {
            $this->redirect('admin.php?page=wpfd-config&error=2');
        }

        $data = $form->sanitize();
        $data = array_map('trim', $data);

        if (!WpfdAddonHelper::saveDropboxConfigs($data)) {
            $this->redirect('admin.php?page=wpfd-config&error=3');
        }

        $this->redirect('admin.php?page=wpfd-config');
    }


    /**
     * Log out
     *
     * @return void
     */
    public function logout()
    {
        $this->dropboxCategory->deleteDropboxCategoryWhenDisconnect();
        $dropbox = new WpfdAddonDropbox();
        $dropbox->logout();
        $this->redirect('admin.php?page=wpfd-config');
    }
    /**
     * Dropbox sync with queue
     *
     * @return void
     */
    public function dropboxsync()
    {
        $wpfdTasks = new \Joomunited\BackgroundTasks\WpfdTasks;

        $wpfdTasks->addDropboxRootToQueue(true);
        $wpfdTasks->runQueue();

        $this->exitStatus(true);
    }
    /**
     * Dropbox sync
     *
     * @return void
     */
    public function dropboxsyncold()
    {
        $dropbox = new WpfdAddonDropbox();
        if (!$dropbox->checkAuth()) {
            $dropboxFolders = $dropbox->listAllFolders();

            if ($dropboxFolders === false) {
                $this->exitStatus(false, 'error during get Dropbox folder list');
            }
            $dataDropboxCategory = get_option('_wpfdAddon_dropbox_category_params');
            $listcates           = array();
            foreach ($dataDropboxCategory as $k => $v) {
                $listcates[$v['id']] = array(
                    'term_id'   => $k,
                    'idDropbox' => $v['idDropbox'],
                    'title'     => $v['idDropboxSlug']
                );
            }

            $diff_del = array_diff_key($listcates, $dropboxFolders);
            $diff_add = array_diff_key($dropboxFolders, $listcates);
            // Check if rename on Dropbox
            if (count($listcates) > 0) {
                $updateName   = false;
                $term_updated = false;
                foreach ($listcates as $folderId => $v) {
                    $termId = $v['term_id'];
                    $term   = get_term($termId, 'wpfd-category');
                    if (is_wp_error($term) || $term === false) {
                        continue;
                    }
                    $newParent    = 0;
                    $fpath        = pathinfo($dropboxFolders[$folderId]['path_lower']);
                    $idDropboxDir = $fpath['dirname'];
                    if (empty($idDropboxDir)) {
                        $idDropboxDir = '/';
                    }
                    if ($idDropboxDir !== '/') {
                        //find parent term
                        $newParent = WpfdAddonHelper::getTermIdByDropboxPath($idDropboxDir);
                    }
                    if ($term && $term->parent !== (int) $newParent) {
                        wp_update_term(
                            $termId,
                            'wpfd-category',
                            array(
                                'parent' => $newParent,
                                'name'   => $dropboxFolders[$folderId]['name']
                            )
                        );
                        $term_updated = true;
                    }
                    if ($term->name !== $dropboxFolders[$folderId]['name']) {
                        wp_update_term(
                            $termId,
                            'wpfd-category',
                            array(
                                'parent' => $newParent,
                                'name'   => $dropboxFolders[$folderId]['name']
                            )
                        );
                        $term_updated = true;
                    }
                    if ($v['title'] !== $dropboxFolders[$folderId]['name']) {
                        $updateName = true;
                        if (!$term_updated) {
                            wp_update_term(
                                $termId,
                                'wpfd-category',
                                array(
                                    'parent' => $newParent,
                                    'name'   => $dropboxFolders[$folderId]['name']
                                )
                            );
                            $term_updated = true;
                        }
                        $dataDropboxCategory[$termId]['idDropbox']     = $dropboxFolders[$folderId]['path_lower'];
                        $dataDropboxCategory[$termId]['idDropboxSlug'] = $dropboxFolders[$folderId]['name'];
                    }
                    if ($v['idDropbox'] !== $dropboxFolders[$folderId]['path_lower']) {
                        $updateName = true;
                        if (!$term_updated) {
                            wp_update_term(
                                $termId,
                                'wpfd-category',
                                array(
                                    'parent' => $newParent,
                                    'name'   => $dropboxFolders[$folderId]['name']
                                )
                            );
                            $term_updated = true;
                        }
                        $dataDropboxCategory[$termId]['idDropbox']     = $dropboxFolders[$folderId]['path_lower'];
                        $dataDropboxCategory[$termId]['idDropboxSlug'] = $dropboxFolders[$folderId]['name'];
                    }
                }
                if ($updateName) {
                    update_option('_wpfdAddon_dropbox_category_params', $dataDropboxCategory, false);
                }
            }

            // Delete categories in WPFD
            if (!empty($diff_del)) {
                $dataDropboxCategory = get_option('_wpfdAddon_dropbox_category_params');
                foreach ($diff_del as $folderId => $v) {
                    $termId = $v['term_id'];
                    //delete term and child term
                    wp_delete_term($termId, 'wpfd-category');
                    //delete option
                    if ($dataDropboxCategory && isset($dataDropboxCategory[$termId])) {
                        unset($dataDropboxCategory[$termId]);
                    }
                }
                update_option('_wpfdAddon_dropbox_category_params', $dataDropboxCategory, false);
            }

            //add new categories
            if (!empty($diff_add)) {
                foreach ($diff_add as $folderId => $folderData) {
                    $fpath        = pathinfo($folderData['path_lower']);
                    $idDropboxDir = $fpath['dirname'];
                    $title        = $folderData['name'];
                    if (empty($idDropboxDir)) {
                        $idDropboxDir = '/';
                    }
                    if ($idDropboxDir !== '/') {
                        //find parent term
                        $parent = (int) WpfdAddonHelper::getTermIdByDropboxPath($idDropboxDir);
                    } else {
                        $parent = 0;
                    }
                    $inserted = wp_insert_term(
                        $title,
                        'wpfd-category',
                        array('parent' => $parent, 'slug' => sanitize_title($title))
                    );
                    if (is_wp_error($inserted)) {
                        // Try again
                        $inserted = wp_insert_term(
                            $title,
                            'wpfd-category',
                            array('parent' => $parent, 'slug' => sanitize_title($title) . '-' . time())
                        );
                        if (is_wp_error($inserted)) {
                            wp_send_json($inserted->get_error_message());
                        }
                    }
                    $lastCats = get_terms(
                        'wpfd-category',
                        'orderby=term_group&order=DESC&hierarchical=0&hide_empty=0&parent=' . $parent . '&number=1'
                    );
                    get_term($inserted['term_id'], 'wpfd-category');
                    if (is_array($lastCats) && count($lastCats)) {
                        $this->updateTermOrder($inserted['term_id'], $lastCats[0]->term_group + 1);
                    }
                    WpfdAddonHelper::updateDefaultCategoryParams($inserted['term_id'], 'dropbox');
                    // Create dropbox category params
                    $dropboxParams = get_option('_wpfdAddon_dropbox_category_params');
                    if (!is_array($dropboxParams)) {
                        $dropboxParams = array();
                    }
                    $dropParams                          = array(
                        'idDropbox'     => $folderData['path_lower'],
                        'termId'        => $inserted['term_id'],
                        'id'            => $folderId,
                        'idDropboxDir'  => $idDropboxDir,
                        'idDropboxSlug' => $fpath['basename']
                    );
                    $dropboxParams[$inserted['term_id']] = $dropParams;
                    update_option('_wpfdAddon_dropbox_category_params', $dropboxParams, false);
                }
            }
            WpfdAddonHelper::setDropboxParam('last_log', date('Y-m-d H:i:s'));
            $this->exitStatus(true);
        }
    }
    /**
     * Delete all OneDrive Business folder
     *
     * @return void
     *
     * @throws Exception Fire message if errors
     */
    public function deleteAllFolder()
    {
        $onedrive = new WpfdAddonDropbox();
        $onedrive->deleteAllFolder();

        wp_send_json_success('ALL_FOLDER_DELETED');
    }
}
