<?php
/**
 * WP File Download
 *
 * @package WP File Download
 * @author  Joomunited
 * @version 1.0
 */

use \Joomunited\WPFramework\v1_0_5\Controller;
use \Joomunited\WPFramework\v1_0_5\Form;
use \Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Model;

defined('ABSPATH') || die();

/**
 * Class WpfdAddonControllerCloud
 */
class WpfdAddonControllerOneDrive extends Controller
{
    /**
     * Onedrive category instance
     *
     * @var boolean
     */
    protected $oneDriveCategory;

    /**
     * WpfdAddonControllerOneDrive constructor.
     */
    public function __construct()
    {
        $app = Application::getInstance('WpfdAddon');
        $path_wpfdhelper = $app->getPath() . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'helpers';
        $path_wpfdhelper .= DIRECTORY_SEPARATOR . 'WpfdHelper.php';
        require_once $path_wpfdhelper;

        $path_wpfdaddononedrive = $app->getPath() . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'classes';
        $path_wpfdaddononedrive .= DIRECTORY_SEPARATOR . 'WpfdAddonOneDrive.php';
        require_once $path_wpfdaddononedrive;

        Application::getInstance('WpfdAddon');
        $this->onedriveCategory = Model::getInstance('onedrivecategory');
    }

    /**
     * Get params config
     *
     * @return array
     */
    public function getParams()
    {
        return WpfdAddonHelper::getAllOneDriveConfigs();
    }

    /**
     * Get params old
     *
     * @return array
     */
    public function getParamsOld()
    {
        return WpfdAddonHelper::getAllOneDriveConfigsOld();
    }

    /**
     * Set params onedrive
     *
     * @param array $data Config data
     *
     * @return void
     */
    public function setParams($data)
    {
        WpfdAddonHelper::saveOneDriveConfigs($data);
    }

    /**
     * Set params old onedrive
     *
     * @param array $data Config data
     *
     * @return void
     */
    public function setParamsOld($data)
    {
        WpfdAddonHelper::saveOneDriveConfigsOld($data);
    }

    /**
     * Get category by onedrive id
     *
     * @param string $oneDriveId Onedrive category id
     *
     * @return boolean|array
     */
    protected function getCategoryByOneDriveId($oneDriveId)
    {
        $termId = WpfdAddonHelper::getTermIdOneDriveByOneDriveId($oneDriveId);
        Application::getInstance('Wpfd');
        $category = Model::getInstance('category');

        return $category->getCategory($termId);
    }

    /**
     * Delete category onedrive
     *
     * @param string $oneDriveId Onedrive category id
     *
     * @return boolean|integer|WP_Error true on success, false if term does not exist. Zero on attempted
     *                             deletion of default Category. WP_Error if the taxonomy does not exist.
     */
    protected function deleteCategoryByOneDriveId($oneDriveId)
    {
        Application::getInstance('WpfdAddon');
        $onedriveCategory = Model::getInstance('onedrivecategory');
        $termId = WpfdAddonHelper::getTermIdOneDriveByOneDriveId($oneDriveId);

        return $onedriveCategory->deleteOneDriveCategory($termId);
    }

    /**
     * Get array onedrive id
     *
     * @return array
     */
    protected function getArrayOneDriveId()
    {
        return WpfdAddonHelper::getAllOneDriveId();
    }

    /**
     * Create category by onedrive
     *
     * @param string  $title      Title
     * @param string  $oneDriveId Onedrive category
     * @param integer $parent     Parent term id
     *
     * @return void
     */
    protected function createCategoryByOneDrive($title, $oneDriveId, $parent)
    {
        Application::getInstance('WpfdAddon');
        $cloudCategory = Model::getInstance('onedrivecategory');
        $cloudCategory->addCategoryFromOneDrive($title, $oneDriveId, $parent);
    }


    /**
     * Update term order
     *
     * @param integer $term_id Term id
     * @param integer $order   Order number
     *
     * @return void
     */
    public function updateTermOrder($term_id, $order)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                'UPDATE ' . $wpdb->terms . ' SET term_group = %d WHERE term_id = %d',
                (int) $order,
                (int) $term_id
            )
        );
    }

    /**
     * Save oneDrive params
     *
     * @return void
     */
    public function saveOneDriveParams()
    {

        $form = new Form();
        $formfile = Application::getInstance('WpfdAddon')->getPath() . DIRECTORY_SEPARATOR . 'admin';
        $formfile .= DIRECTORY_SEPARATOR . 'forms' . DIRECTORY_SEPARATOR . 'onedrive_config.xml';
        if (!$form->load($formfile)) {
            $this->redirect('admin.php?page=wpfd-config&error=1');
        }
        if (!$form->validate()) {
            $this->redirect('admin.php?page=wpfd-config&error=2');
        }
        $data = $form->sanitize();
        $data = array_map('trim', $data);
        if (!WpfdAddonHelper::saveOneDriveConfigs($data)) {
            $this->redirect('admin.php?page=wpfd-config&error=3');
        }
        $this->redirect('admin.php?page=wpfd-config');
    }


    /**
     * Get folder onedrive in WPFD
     *
     * @return array
     */
    public function foldersOneDriveInWPFD()
    {
        $catOneDrive = array();
        $onedriveParams = WpfdAddonHelper::getAllOneDriveParams();
        if ($onedriveParams['onedrive']) {
            Application::getInstance('Wpfd');
            $category = Model::getInstance('category');
            foreach ($onedriveParams['onedrive'] as $key) {
                $categoryOneDrive = $category->getCategory($key['termId']);
                $oneDriveId = $key['oneDriveId'];
                $catOneDrive[$oneDriveId] = array('title' => $categoryOneDrive->name,
                    'parent_id' => $categoryOneDrive->parent);
            }
        } else {
            return $catOneDrive;
        }
        return $catOneDrive;
    }

    /**
     * Get folder in onedrive
     *
     * @return array|boolean
     */
    public function foldersInOneDrive()
    {
        $data = $this->getParams();
        $onedrive = new WpfdAddonOneDrive();
        $lstFolder = $onedrive->getListFolder($data['onedriveBaseFolder']['id']);
        return $lstFolder;
    }

    /**
     * Sync Onedrive using new queue
     *
     * @return void
     */
    public function onedrivesync()
    {
        $wpfdTasks = new \Joomunited\BackgroundTasks\WpfdTasks;

        $wpfdTasks->addOnedriveRootToQueue(true);
        $wpfdTasks->runQueue();
        $this->exitStatus(true);
    }
    /**
     * Onedrive sync
     *
     * @return void
     */
//    public function onedrivesyncold()
//    {
//        $params = $this->getParams();
//
//        if (isset($params['onedriveConnected']) && $params['onedriveConnected'] === 1) {
//            Application::getInstance('Wpfd');
//            $category = Model::getInstance('category');
//            // Folders in WPFD
//            $folderOneDriveInWPFD = $this->foldersOneDriveInWPFD();
//            // Folders in OneDrive Drive
//            $folderCloudInOneDrive = $this->foldersInOneDrive();
//
//            // Folders created in OneDrive without have in WPFD
//            $folders_diff = array();
//            // Folders will delete
//            $folders_diff_del = array();
//            if ($folderCloudInOneDrive !== false) {
//                // To ensure there isn't error when connect with GD
//                if (count($folderOneDriveInWPFD) > 0) {
//                    // Change folders title
//                    foreach ($folderOneDriveInWPFD as $k => $v) {
//                        if (!isset($folderOneDriveInWPFD[$k])) {
//                            continue;
//                        }
//                        if ($folderOneDriveInWPFD[$k]['title'] !== $folderCloudInOneDrive[$k]['title']) {
//                            $objectCurrent = $this->getCategoryByOneDriveId($k);
//                            try {
//                                $category->saveTitle($objectCurrent->term_id, $folderCloudInOneDrive[$k]['title']);
//                            } catch (Exception $e) {
//                                $this->exitStatus(false, 'updateTitleById-Exception: ' . $e->getMessage());
//                            }
//                        }
//                        // Correcting the parent of local folder with cloud folders
//                        $localParentCloudId =  WpfdAddonHelper::getOneDriveIdByTermId($folderOneDriveInWPFD[$k]['parent_id']);
//                        if ($localParentCloudId !== $folderCloudInOneDrive[$k]['parent_id']) {
//                            // Get new parent term_id
//                            $newLocalParentTermId = WpfdAddonHelper::getTermIdOneDriveByOneDriveId($folderCloudInOneDrive[$k]['parent_id']);
//                            // Update local parent
//                            if ($newLocalParentTermId) {
//                                $currentFolderTermId = WpfdAddonHelper::getTermIdOneDriveByOneDriveId($k);
//                                if ($currentFolderTermId) {
//                                    wp_update_term($currentFolderTermId, 'wpfd-category', array('parent' => $newLocalParentTermId));
//                                    $folderOneDriveInWPFD[$k]['parent_id'] = $newLocalParentTermId;
//                                }
//                            }
//                        }
//                    }
//                    $folders_diff = array_diff_key($folderCloudInOneDrive, $folderOneDriveInWPFD);
//                    $folders_diff_del = array_diff_key($folderOneDriveInWPFD, $folderCloudInOneDrive);
//                } else {
//                    $folders_diff = $folderCloudInOneDrive;
//                }
//            } elseif ($folderCloudInOneDrive === false) {
//                // An error come
//                $this->exitStatus(false, 'Something wrong! Please check your Onedrive credentials!');
//            }
//            // Delete folders in dropfile
//            if (count($folders_diff_del) > 0) {
//                foreach ($folders_diff_del as $OnediveIdDel => $folderDataDel) {
//                    $this->deleteCategoryByOneDriveId($OnediveIdDel);
//                }
//            }
//            // If exists diff key array
//            if (count($folders_diff) > 0) {
//                $lstOneDriveIdOnWPFD = $this->getArrayOneDriveId();
//                foreach ($folders_diff as $onedriverId => $folderData) {
//                    try {
//                        // If has parent_id.
//                        if ($folderData['parent_id'] === 0) {
//                            // Create Folder New
//                            $this->createCategoryByOneDrive($folderData['title'], $onedriverId, 0);
//                            $lstOneDriveIdOnWPFD[] = $onedriverId;
//                        } else {
//                            $check = in_array($folderData['parent_id'], $lstOneDriveIdOnWPFD);
//                            if (!$check) {
//                                // Create Parent New
//                                $ParentOneDriveInfo = $folderCloudInOneDrive[$folderData['parent_id']];
//                                $this->createCategoryByOneDrive(
//                                    $ParentOneDriveInfo['title'],
//                                    $folderData['parent_id'],
//                                    0
//                                );
//                                $lstOneDriveIdOnWPFD[] = $folderData['parent_id'];
//                                // Create Children New with parent_id in WPFD
//                                if ($this->getCategoryByOneDriveId($folderData['parent_id'])) {
//                                    $catRecentCreate = $this->getCategoryByOneDriveId($folderData['parent_id']);
//                                    $this->createCategoryByOneDrive(
//                                        $folderData['title'],
//                                        $onedriverId,
//                                        $catRecentCreate->term_id
//                                    );
//                                }
//                            } else {
//                                // Create Children New with parent_id in WPFD
//                                $catOldInfo = $this->getCategoryByOneDriveId($folderData['parent_id']);
//                                $this->createCategoryByOneDrive(
//                                    $folderData['title'],
//                                    $onedriverId,
//                                    $catOldInfo->term_id
//                                );
//                                $lstOneDriveIdOnWPFD[] = $onedriverId;
//                            }
//                        }
//                    } catch (Exception $e) {
//                        $erros = $e->getMessage();
//                        WpfdAddonHelper::writeLog($erros);
//                        $this->exitStatus(false, 'Something wrong! ' . $erros);
//                        break;
//                    }
//                }
//            }
//            WpfdAddonHelper::setOneDriveParam('last_log', date('Y-m-d H:i:s'));
//            $this->exitStatus(true);
//        }
//    }

    /**
     * Authenticate
     *
     * @return void
     */
//    public function authenticate()
//    {
//        $onedrive = new WpfdAddonOneDrive();
//        $credentials = $onedrive->authenticate();
//        $onedrive->storeCredentials($credentials);
//        $data = $this->getParams();
//        $dataOld = $this->getParamsOld();
//        $onedriveClientIdOld = isset($dataOld['onedriveKey']) ? $dataOld['onedriveKey'] : 0;
//        $onedriveClientId = isset($data['onedriveKey']) ? $data['onedriveKey'] : -1;
//        $baseFolderIdOld = isset($dataOld['onedriveBaseFolder']) ? $dataOld['onedriveBaseFolder']['id'] : -1;
//        $baseFolderNameOld = isset($dataOld['onedriveBaseFolder']) ? $dataOld['onedriveBaseFolder']['name'] : -1;
//        if ($dataOld && $onedriveClientId === $onedriveClientIdOld) {
//            if ($onedrive->getFolder($baseFolderIdOld, $baseFolderNameOld)) {
//                $data['onedriveBaseFolder'] = $dataOld['onedriveBaseFolder'];
//            } else {
//                $newentry = $this->newentryOnedrive($onedrive);
//                $idroot = $newentry->getId();
//                $nameroot = $newentry->getName();
//                $data['onedriveBaseFolder'] = array(
//                    'id' => WpfdAddonHelper::replaceIdOneDrive($idroot), 'name' => $nameroot);
//                $this->setParamsOld($data);
//            }
//        } else {
//            $newentry = $this->newentryOnedrive($onedrive);
//            $idroot = $newentry->getId();
//            $nameroot = $newentry->getName();
//            $data['onedriveBaseFolder'] = array(
//                'id' => WpfdAddonHelper::replaceIdOneDrive($idroot), 'name' => $nameroot);
//            $this->setParamsOld($data);
//        }
//        $data['onedriveConnected'] = 1;
//        $this->setParams($data);
//        $this->redirect('admin.php?page=wpfdAddon-onedrive&task=onedrive.authenticated');
//    }

    /**
     * Authenticated
     *
     * @return void
     */
    public function authenticated()
    {
        $view = $this->loadView();
        $view->render('redirect');
    }

    /**
     * New root folder in onedrive
     *
     * @param WpfdAddonOneDrive $onedrive WpfdAddonOneDrive instance
     *
     * @return OneDrive_Service_Drive_Item
     */
//    public function newentryOnedrive($onedrive)
//    {
//        $newentry = $onedrive->addFolderRoot('WP File Download - ' . get_bloginfo('name'));
//        $decoded = json_decode($newentry['responsebody'], true);
//
//        return new OneDrive_Service_Drive_Item($decoded);
//    }

    /**
     * Log out
     *
     * @return void
     */
    public function logout()
    {
        Application::getInstance('WpfdAddon');
        $onedriveCategory = Model::getInstance('onedrivebusinesscategory');
        $onedriveCategory->deleteCategoryWhenDisconnect();
        $onedrive = new WpfdAddonOneDrive();
        $onedrive->logout();
        $data                      = $this->getParams();
        $data['onedriveConnected'] = 0;
        $this->setParams($data);
        $this->redirect('admin.php?page=wpfd-config');
    }

    /**
     * Delete all OneDrive folder
     *
     * @return void
     *
     * @throws Exception Fire message if errors
     */
    public function deleteAllFolder()
    {
        $onedrive = new WpfdAddonOneDrive();
        $onedrive->deleteAllFolder();

        wp_send_json_success('ALL_FOLDER_DELETED');
    }
}
