<?php
use Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Model;
use Joomunited\WPFramework\v1_0_5\Utilities;

/**
 * OneDrive Business
 */
add_filter('wpfdAddonShowCategoryOneDriveBusiness', 'displayCategoriesOneDriveBusiness', 10, 1);
add_filter('wpfdAddonDownloadOneDriveBusinessInfo', 'wpfdAddonDownloadOneDriveBusinessInfo', 10, 1);
add_action('wpfdAddonCopyOneDriveBusiness', 'wpfdAddonCopyOneDriveBusiness', 10, 2);
add_action('wpfdAddonMoveOneDriveBusiness', 'wpfdAddonMoveOneDriveBusiness', 10, 2);
add_filter('wpfdAddonOneDriveBusinessChangeOrder', 'wpfdAddonOneDriveBusinessChangeOrder', 10, 1);
add_action('wp_ajax_onedriveBusinessSync', 'wpfdAddonOneDriveBusinessSync');
add_action('wp_ajax_nopriv_onedriveBusinessSync', 'wpfdAddonOneDriveBusinessSync');
add_action('wpfd_addon_sync_buttons', 'displayOneDriveBusinessButtonSync', 30);
add_filter('wpfdAddonUploadOneDriveBusinessVersion', 'wpfdAddonUploadOneDriveBusinessVersion', 10, 2);

// Push notifications ajax
add_action('wp_ajax_onedriveBusinessWatchChanges', 'wpfdAddonOnedriveBusinessWatchChanges_callback');
add_action('wp_ajax_nopriv_onedriveBusinessWatchChanges', 'wpfdAddonOnedriveBusinessWatchChanges_callback');

add_action('wp_ajax_onedriveBusinessPushListener', 'wpfdAddonOnedriveBusinessPushListener_callback');
add_action('wp_ajax_nopriv_onedriveBusinessPushListener', 'wpfdAddonOnedriveBusinessPushListener_callback');

add_action('admin_init', function () {
    $state = Utilities::getInput('state', 'GET', 'string');
    if ($state === 'wpfd-onedrive-business') {
        $onedrive    = new WpfdAddonOneDriveBusiness();
        $onedrive->authenticate();
    }
});

/**
 * Display categories onedrive business
 *
 * @param integer $termId Wordpress taxonomy id
 *
 * @return string
 */
function displayCategoriesOneDriveBusiness($termId)
{
    $cloudType = get_term_meta($termId, 'wpfd_drive_type', true);
    if ($cloudType === 'onedrive_business') {
        return '<i class="onedrive-business-icon"></i>&nbsp';
    }

    return '';
}

/**
 * Get list file from onedriver
 *
 * @param integer $termId      Term Id
 * @param boolean $listIdFlies List id Files?
 *
 * @return array
 */
//function wpfdAddonDisplayOneDriveBusinessCategories($termId, $listIdFlies = false)
//{
//    if (WpfdAddonHelper::getOneDriveBusinessIdByTermId($termId)) {
//        Application::getInstance('WpfdAddon');
//        $onedriveFile = Model::getInstance('onedrivebusinessfiles');
//
//        return $onedriveFile->getListOneDriveBusinessFiles($termId, $listIdFlies);
//    }
//}

/**
 * Get OneDrive business file info
 *
 * @param string  $idFile     File id
 * @param integer $idCategory Term id
 *
 * @return boolean|array array File if success
 *                       false if error
 */
function wpfdAddonOneDriveBusinessGetFileInfo($idFile, $idCategory)
{
    Application::getInstance('WpfdAddon');
    $onedriveFile = Model::getInstance('onedrivebusinessfiles');
    /* @var $onedriveFile wpfdAddonModelOnedriveBusinessFiles */
    return $onedriveFile->getOneDriveBusinessFileInfo($idFile, $idCategory);
}
/**
 * Save OneDrive file Info
 *
 * @param array   $data       File data
 * @param integer $idCategory Term id
 *
 * @return string File id
 */
function wpfdAddonSaveOneDriveBusinessFileInfo($data, $idCategory)
{
    Application::getInstance('WpfdAddon');
    $onedriveFile = Model::getInstance('onedrivebusinessfiles');
    return $onedriveFile->saveOneDriveBusinessFileInfo($data, $idCategory);
}

/**
 * Upload OneDrive file
 *
 * @param array   $file         File info
 * @param string  $file_current File path
 * @param integer $id_category  Term id
 *
 * @return void
 */
function wpfdAddonUploadOneDriveBusinessFile($file, $file_current, $id_category)
{
    $onedrive = new WpfdAddonOneDriveBusiness();
    $pic = array();
    $pic['error'] = 0;
    $pic['name'] = $file['title'] . '.' . $file['ext'];
    $pic['type'] = '';
    $pic['size'] = $file['size'];
    $result = $onedrive->uploadFile(
        $file['title'] . '.' . $file['ext'],
        $pic,
        $file_current,
        WpfdAddonHelper::getOneDriveBusinessIdByTermId($id_category)
    );

    // Add pending uploaded file
    if (!function_exists('wpfd_can_edit_own_category')
        || !function_exists('wpfd_user_is_owner_of_category')) {
        $app = Application::getInstance('Wpfd');
        $path_wpfdfunctions = $app->getPath() . DIRECTORY_SEPARATOR . 'functions.php';
        require_once $path_wpfdfunctions;
    }

    $uploadFrom  = Utilities::getInput('upload_from') ? Utilities::getInput('upload_from') : '';
    $publishFile = false;

    if ((int)get_current_user_id() !== 0) {
        if (wpfd_can_edit_category()) {
            $publishFile = true;
        }

        if (wpfd_can_edit_own_category()) {
            Application::getInstance('Wpfd');
            $categoryModel                  = Model::getInstance('category');
            $categoryOnedriveBusiness       = $categoryModel->getCategory($id_category);
            if (wpfd_user_is_owner_of_category($categoryOnedriveBusiness)) {
                $onedriveBusinessCategoryInfo = get_term($id_category, 'wpfd-category');
                if (!empty($onedriveBusinessCategoryInfo) && !is_wp_error($onedriveBusinessCategoryInfo)) {
                    if ($onedriveBusinessCategoryInfo->description !== 'null' && $onedriveBusinessCategoryInfo->description !== '') {
                        $cateParams = json_decode($onedriveBusinessCategoryInfo->description, true);
                        if (isset($cateParams['category_own'])
                            && (int)$cateParams['category_own'] === (int)$categoryOnedriveBusiness->params['category_own']) {
                            $publishFile = true;
                        }
                    }
                }
            }
        }
    }

    if (isset($result['file']['id']) && $uploadFrom === 'front' && $publishFile === false) {
        // Save values to set pending files
        $fileId                  = $result['file']['id'];
        $pendingFiles            = get_option('_wpfdAddon_upload_pending_files') ? get_option('_wpfdAddon_upload_pending_files') : array();
        $pendingFiles[$fileId]   = get_current_user_id();
        update_option('_wpfdAddon_upload_pending_files', $pendingFiles, false);
    }

    add_filter('wpfd_addon_onedrive_business_uploaded_result', function () use ($result) {
        return $result['file'];
    });
}
/**
 * Copy onedrive
 *
 * @param string  $id_file     File id
 * @param integer $id_category Term id
 *
 * @return void
 */
function wpfdAddonCopyOneDriveBusiness($id_file, $id_category)
{
    $oneDrive = new WpfdAddonOneDriveBusiness();
    $result = $oneDrive->copyFile($id_file, WpfdAddonHelper::getOneDriveBusinessIdByTermId($id_category));

    add_filter('wpfd_addon_onedrive_business_uploaded_result', function () use ($result) {
        return $result;
    });
}

/**
 * Copy onedrive
 *
 * @param string  $id_file     File id
 * @param integer $id_category Term id
 *
 * @return void
 */
function wpfdAddonMoveOneDriveBusiness($id_file, $id_category)
{
    $oneDrive = new WpfdAddonOneDriveBusiness();
    $result = $oneDrive->moveFile($id_file, WpfdAddonHelper::getOneDriveBusinessIdByTermId($id_category));

    add_filter('wpfd_addon_onedrive_business_uploaded_result', function () use ($result) {
        return $result;
    });
}

/**
 * Update Version and Description of Onedrive Business on copy
 *
 * @param string  $version     Version
 * @param string  $description Description
 * @param integer $catId       Category term id
 *
 * @return void
 */
function wpfdAddonOnedriveBusinessUpdateVersionDescription($version, $description, $catId)
{
    $uploadedFile = array();
    /**
     * Filter to get uploaded dropbox file params
     *
     * @param array
     */
    $uploadedFile = apply_filters('wpfd_addon_onedrive_business_uploaded_result', $uploadedFile);

    if (empty($uploadedFile) || !isset($uploadedFile['id'])) {
        return;
    }

    $new_file_id = $uploadedFile['id'];
    // Update file version and description
    $fileInfos = WpfdAddonHelper::getOnedriveBusinessFileInfos();

    if (!empty($fileInfos)) {
        if (isset($fileInfos[$catId]) && isset($fileInfos[$catId][$new_file_id])) {
            $fileInfos[$catId][$new_file_id]['version'] = isset($version) ? $version : '';
            $fileInfos[$catId][$new_file_id]['description'] = isset($description) ? $description : '';
        } else {
            $fileInfos[$catId] = array(
                $new_file_id => array(
                    'version'     => isset($version) ? $version : '',
                    'description' => isset($description) ? $description : ''
                )
            );
        }
    }

    WpfdAddonHelper::setOneDriveBusinessFileInfos($fileInfos);
}

/**
 * Delete file from folder
 *
 * @param integer $idCategory Term id
 * @param string  $idFile     File id
 *
 * @return boolean true if success
 *                 false on error
 */
function wpfdAddonDeleteOneDriveBusinessFiles($idCategory, $idFile)
{
    Application::getInstance('WpfdAddon');
    $onedriveFile = Model::getInstance('onedrivebusinessfiles');
    return $onedriveFile->deleteOneDriveBusinessFile($idCategory, $idFile);
}

/**
 * Download OneDrive business info
 *
 * @param string $id_file File id
 *
 * @return boolean|stdClass|WP_Error false on response code other than 200
 *                             stdClass File info when success
 *                             WP_Error on Exception
 */
function wpfdAddonDownloadOneDriveBusinessInfo($id_file)
{
    $onedriveFile = new WpfdAddonOneDriveBusiness();
    return $onedriveFile->downloadFile($id_file);
}

/**
 * OndDrive change order
 *
 * @param integer $pk Term move order
 *
 * @return void
 */
function wpfdAddonOneDriveBusinessChangeOrder($pk)
{
    Application::getInstance('WpfdAddon');
    $OnedriveCat = Model::getInstance('onedrivebusinesscategory');
    $OnedriveCat->changeOrder($pk);
}
/**
 * Set category oneDrive title
 *
 * @param integer $termId Term id
 * @param string  $title  Title
 *
 * @return boolean true if success
 *                 false if error
 */
function wpfdAddonSetCategoryOneDriveBusinessTitle($termId, $title)
{
    Application::getInstance('WpfdAddon');
    $onedriveCategory = Model::getInstance('onedrivebusinesscategory');
    return $onedriveCategory->changeCategoryName(WpfdAddonHelper::getOneDriveBusinessIdByTermId($termId), $title);
}

/**
 * OneDrive Sync callback
 *
 * @return void
 */
function wpfdAddonOneDriveBusinessSync()
{
    wpfdAddon_call('onedrivebusiness.onedrivebusinesssync');
}
/**
 * Display OneDrive Button Sync
 *
 * @return void
 */
function displayOneDriveBusinessButtonSync()
{
    $data = WpfdAddonHelper::getAllOneDriveBusinessConfigs();
    $onedriveConnected = isset($data['connected']) ? $data['connected'] : 0;
    if ($onedriveConnected) {
        echo ' <a href="#" title="' . esc_html__('Sync with OneDrive Business', 'wpfdAddon') . '" id="btn-sync-onedrive-business"  
                data-loading-text="<i class=\'wpfd-svg-icon-sync-loading\'></i>"  
                data-complete-text="<i class=\'wpfd-svg-icon-sync-onedrive\'></i>" class="ju-button">
                 <i class="wpfd-svg-icon-sync-onedrive"></i>
               </a>';
    }
}
/**
 * Uplaod onedrive version
 *
 * @param array   $data   File data
 * @param integer $termId Term id
 *
 * @return boolean
 */
function wpfdAddonUploadOneDriveBusinessVersion($data, $termId)
{
    Application::getInstance('WpfdAddon');
    $dropFile = Model::getInstance('onedrivebusinessfiles');
    return $dropFile->uploadOneDriveBusinessVersion($data, $termId);
}
/**
 * Toggle watch changes
 *
 * @return void
 */
function wpfdAddonOnedriveBusinessWatchChanges_callback()
{
    wpfdAddon_call('onedrivebusiness.onedriveBusinessStopWatchChanges');
}

/**
 * Webhook to listen changes form Google Drive
 *
 * @return void
 */
function wpfdAddonOnedriveBusinessPushListener_callback()
{
    wpfdAddon_call('onedrivebusiness.listener');
}

/**
 * WpfdAddonOnedriveBusinessPendingUploadFiles
 *
 * @param integer|string $categoryId   Category id
 * @param array          $pendingFiles Upload file list
 *
 * @throws Exception                        Fire message if errors
 *
 * @return void
 */
function wpfdAddonOnedriveBusinessPendingUploadFiles($categoryId, $pendingFiles)
{
    Application::getInstance('WpfdAddon');
    $onedriveFile   = Model::getInstance('onedrivebusinessfiles');
    $uploadedUsers  = get_option('_wpfdAddon_onedrive_business_file_user_upload') ? get_option('_wpfdAddon_onedrive_business_file_user_upload') : array();
    if (!empty($pendingFiles)) {
        foreach ($pendingFiles as $key => $value) {
            $fileId     = $key;
            $fileInfo   = $onedriveFile->getOneDriveBusinessFileInfo($fileId, $categoryId);
            if (!empty($fileInfo)) {
                $fileInfo['state'] = '0';
                $onedriveFile->saveOneDriveBusinessFileInfo($fileInfo, $categoryId);
            }
            $uploadedUsers[$fileId] = get_current_user_id();
        }

        delete_option('_wpfdAddon_upload_pending_files');
        update_option('_wpfdAddon_onedrive_business_file_user_upload', $uploadedUsers, false);
    }

    wp_send_json(array('success' => true, 'data' => $pendingFiles));
}

/**
 * Publish onedrivebusiness files
 *
 * @param array   $fileIds    File list info
 * @param integer $categoryId Term id
 *
 * @throws Exception               Fire if errors
 *
 * @return void
 */
function wpfdAddonOnedriveBusinessPublishFile($fileIds, $categoryId)
{
    Application::getInstance('WpfdAddon');
    $onedrivebusinessFile   = Model::getInstance('onedrivebusinessfiles');
    $publishedFileIds       = array();
    foreach ($fileIds as $fileId) {
        $fileInfo   = $onedrivebusinessFile->getOneDriveBusinessFileInfo($fileId, $categoryId);
        if (!empty($fileInfo)) {
            $fileInfo['state'] = '1';
            $onedrivebusinessFile->saveOneDriveBusinessFileInfo($fileInfo, $categoryId);
        }
        // Remove pending status
        $uploadUsers = get_option('_wpfdAddon_onedrive_business_file_user_upload');
        if (is_array($uploadUsers) && !empty($uploadUsers)) {
            if (array_key_exists((string) $fileId, $uploadUsers)) {
                update_option('_wpfdAddon_onedrive_business_file_user_upload', $uploadUsers, false);
                unset($uploadUsers[$fileId]);
            }
        }

        $publishedFileIds[] = $fileId;
    }

    if (!empty($publishedFileIds)) {
        wp_send_json(array('success' => true, 'data' => $publishedFileIds));
    } else {
        wp_send_json(array('success' => false, 'data' => array()));
    }
}

/**
 * Unpublish onedrivebusiness files
 *
 * @param array   $fileIds    File list info
 * @param integer $categoryId Term id
 *
 * @throws Exception               Fire if errors
 *
 * @return void
 */
function wpfdAddonOnedriveBusinessUnpublishFile($fileIds, $categoryId)
{
    Application::getInstance('WpfdAddon');
    $onedrivebusinessFile     = Model::getInstance('onedrivebusinessfiles');
    $unpublishedFileIds       = array();
    foreach ($fileIds as $fileId) {
        $fileInfo   = $onedrivebusinessFile->getOneDriveBusinessFileInfo($fileId, $categoryId);
        if (!empty($fileInfo)) {
            $fileInfo['state'] = '0';
            $onedrivebusinessFile->saveOneDriveBusinessFileInfo($fileInfo, $categoryId);
        }

        $unpublishedFileIds[] = $fileId;
    }

    if (!empty($unpublishedFileIds)) {
        wp_send_json(array('success' => true, 'data' => $unpublishedFileIds));
    } else {
        wp_send_json(array('success' => false, 'data' => array()));
    }
}
