<?php
use Joomunited\WPFramework\v1_0_5\Application;
use Joomunited\WPFramework\v1_0_5\Model;

if (!class_exists('WpfdAddonHelper')) {
    require_once WPFDA_PLUGIN_DIR_PATH . 'app/admin/helpers/WpfdHelper.php';
}
if (!class_exists('WpfdAddonGoogle')) {
    require_once WPFDA_PLUGIN_DIR_PATH . 'app/admin/classes/WpfdAddonGoogle.php';
}
// Check if google drive connected
$cloudConfig = WpfdAddonHelper::getAllCloudConfigs();
$googleDrive = new WpfdAddonGoogleDrive();
$isAuthenticated = $googleDrive->checkAuth();
if (isset($cloudConfig['googleCredentials']) && (!empty($cloudConfig['googleCredentials'])) && $isAuthenticated) {
    add_filter('wpfdAddonCategoryFrom', 'wpfdAddonCategoryFrom', 10, 1);
    add_filter('wpfdAddonGetListGoogleDriveFile', 'wpfdAddonGetListGoogleDriveFile', 10, 5);
    add_filter('wpfdAddonGetGoogleDriveFile', 'wpfdAddonGetGoogleDriveFile', 10, 3);
    add_filter('wpfdAddonDownloadGoogleDriveFile', 'wpfdAddonDownloadGoogleDriveFile', 10, 1);
    add_filter('wpfd_level_category', 'wpfdAddonLevelCategory', 9, 1);
    add_filter('wpfdAddonSearchCloud', 'wpfdAddonSearchCloud', 10, 2);
    add_filter('wpfdAddonDownloadCheckGoogleDriveCategory', 'wpfdAddonDownloadCheckGoogleDriveCategory', 10, 2);
}

/**
 * Get list google drive files
 *
 * @param integer $termId       Term id
 * @param string  $ordering     Ordering
 * @param string  $orderingdir  Ordering direction
 * @param string  $categorySlug Category slug
 * @param string  $token        Security token
 *
 * @return mixed
 */
function wpfdAddonGetListGoogleDriveFile($termId, $ordering = 'title', $orderingdir = 'asc', $categorySlug = '', $token = '')
{
    $googleCate = new WpfdAddonGoogleDrive;
    $listFiles = $googleCate->listFiles(WpfdAddonHelper::getGoogleDriveIdByTermId($termId), $ordering, $orderingdir);
    return wpfdAddonAddInfosToFiles($listFiles, $termId, $categorySlug, $token);
}


/**
 * Get a file in google drive
 *
 * @param string  $fileId     Google file id
 * @param integer $categoryId Term id
 * @param string  $token      Security token
 *
 * @return array|boolean
 */
function wpfdAddonGetGoogleDriveFile($fileId, $categoryId, $token)
{
    $googleCate = new WpfdAddonGoogleDrive;
    return $googleCate->getFileInfos($fileId, null, $categoryId, $token);
}


/**
 * Download googleDrive file
 *
 * @param string $fileId Google file id
 *
 * @return boolean|stdClass
 */
function wpfdAddonDownloadGoogleDriveFile($fileId)
{
    $googleCate = new WpfdAddonGoogleDrive;
    $googleCate->incrHits($fileId);
    $result = $googleCate->download($fileId);

    return $result;
}


/**
 * Get category in dropbox
 *
 * @param WP_Term $category Category
 *
 * @return mixed
 */
function wpfdAddonLevelCategoryDropbox($category)
{
    $drop_id = WpfdAddonHelper::getDropboxIdByTermId($category->term_id);
    if ($drop_id) {
        $category->wp_term_id = $category->term_id;
        $category->wp_parent = $category->parent;
        $category->term_id = $drop_id;
        $category->parent = WpfdAddonHelper::getDropboxIdByTermId($category->parent);
    }
    return $category;
}


/**
 * Search cloud
 *
 * @param array $cloud_cond Cloud condition to search
 * @param array $filters    Filters
 *
 * @return array
 */
function wpfdAddonSearchCloud($cloud_cond, $filters)
{
    $googleCate  = new WpfdAddonGoogleDrive;
    $cloudConfig = WpfdAddonHelper::getAllCloudConfigs();
    $excludes = explode(',', trim($filters['exclude']));

    if ($filters['catid'] !== '' && is_string($filters['catid'])) {
        $q_tmp   = " '" . $filters['catid'] . "' in parents";
        $folders = $googleCate->getListFolder($filters['catid']);
        if (count($folders)) {
            $q_tmp = '(' . $q_tmp;
            foreach ($folders as $id => $folder) {
                $q_tmp .= " or '" . $id . "' in parents ";
            }
            $q_tmp .= ')';
        }
    } else {
        $foldersQuery = $googleCate->searchcondition($cloudConfig);
        if ($foldersQuery !== false) {
            $q_tmp = '(' . $foldersQuery . ')';
        } else {
            $q_tmp = '( "' . $cloudConfig['googleBaseFolder'] . '" in parents)';
        }
    }
    $cloud_cond[] = $q_tmp;
    $q            = implode(' and ', $cloud_cond);
    $results      = $googleCate->getAllFilesInAppFolder($q);

    Application::getInstance('Wpfd');
    $modelConfig = Model::getInstance('configfront');
    $modelToken = Model::getInstance('tokens');
    $params      = $modelConfig->getGlobalConfig();
    $config      = get_option('_wpfd_global_config');
    if (empty($config) || empty($config['uri'])) {
        $seo_uri = 'download';
    } else {
        $seo_uri = rawurlencode($config['uri']);
    }

    if (empty($config) || !isset($config['rmdownloadext'])) {
        $rmdownloadext = false;
    } else {
        $rmdownloadext = ((int) $config['rmdownloadext'] === 1) ? true : false;
    }

    $viewer_type      = WpfdBase::loadValue($params, 'use_google_viewer', 'lightbox');
    $perlink          = get_option('permalink_structure');
    $rewrite_rules    = get_option('rewrite_rules');
    $extension_list   = 'png,jpg,pdf,ppt,pptx,doc,docx,xls,xlsx,dxf,ps,eps,xps,psd,tif,tiff,bmp,svg,pages,ai,dxf,ttf,txt,mp3,mp4';
    $extension_viewer = explode(',', WpfdBase::loadValue($params, 'extension_viewer', $extension_list));
    $extension_viewer = array_map('trim', $extension_viewer);

    if (is_array($results)) {
        foreach ($results as $key => $value) {
            $parentInfo      = $googleCate->getParentInfo($value->id);
            $value->cattitle = $parentInfo['title'];
            $value->catname  = $parentInfo['title'];
            $value->catid    = WpfdAddonHelper::getCatIdByCloudId($parentInfo['id']);

            if (isset($value->parent) && in_array((string) $value->parent->id, $excludes)) {
                unset($results[$key]);
                continue;
            }

            $value->created  = mysql2date(
                WpfdBase::loadValue($params, 'date_format', get_option('date_format')),
                $value->created_time
            );
            $value->modified = mysql2date(
                WpfdBase::loadValue($params, 'date_format', get_option('date_format')),
                $value->modified_time
            );
            $category        = get_term($value->catid, 'wpfd-category');

            if (!empty($rewrite_rules)) {
                if (strpos($perlink, 'index.php')) {
                    $linkdownload = get_site_url() . '/index.php/' . $seo_uri . '/' . $value->catid . '/';
                    $linkdownload .= $category->slug . '/' . $value->id . '/' . $value->post_title;
                    $value->linkdownload = $linkdownload;
                } else {
                    $linkdownload = get_site_url() . '/' . $seo_uri . '/' . $value->catid . '/' . $category->slug;
                    $linkdownload .= '/' . $value->id . '/' . $value->post_title;
                    $value->linkdownload = $linkdownload;
                }
                if ($value->ext && !$rmdownloadext) {
                    $value->linkdownload .= '.' . $value->ext;
                }
            } else {
                $linkdownload = admin_url('admin-ajax.php') . '?juwpfisadmin=false&action=wpfd&task=file.download';
                $linkdownload .= '&wpfd_category_id=' . $value->catid . '&wpfd_file_id=' . $value->id;
                $value->linkdownload = $linkdownload;
            }
            $remote_url = (isset($file_meta) && isset($file_meta['remote_url']) && ((int) $file_meta['remote_url'] === 1)) ? true : false;
            if ($viewer_type !== 'no' &&
                in_array($value->ext, $extension_viewer) &&
                ($remote_url === false)) {
                $value->viewer_type = $viewer_type;
                $token = $modelToken->getOrCreateNew();
                $value->viewerlink = WpfdHelperFile::isMediaFile($value->ext) ?
                    WpfdHelperFile::getMediaViewerUrl($value->id, $value->catid, $value->ext, $token)
                    : WpfdHelperFile::getViewerUrl($value->id, $value->catid, $token);
            }
        }
    } else {
        $results = array();
    }
    return $results;
}


/**
 * Check if category and file id are correct
 *
 * @param string $termId Term id
 * @param string $fileId File id
 *
 * @return boolean
 */
function wpfdAddonDownloadCheckGoogleDriveCategory($termId, $fileId)
{
    $cloud_id = WpfdAddonHelper::getGoogleDriveIdByTermId($termId);
    $googleCate = new WpfdAddonGoogleDrive;
    if ($googleCate->checkFileFolderValid($fileId, $cloud_id)) {
        return $termId;
    }
    return false;
}
/**
 * Add info to file
 *
 * @param array   $files                    Files
 * @param integer $termId                   Term id
 * @param string  $googleDriveCategoryTitle Google drive category title
 * @param string  $token                    Security Token
 *
 * @return mixed
 */
function wpfdAddonAddInfosToFiles($files, $termId, $googleDriveCategoryTitle, $token)
{
    Application::getInstance('Wpfd');
    $modelConfig = Model::getInstance('configfront');
    if (method_exists($modelConfig, 'getGlobalConfig')) {
        $params = $modelConfig->getGlobalConfig();
    } else {
        $params = $modelConfig->getConfig();
    }

    $config      = get_option('_wpfd_global_config');
    if (empty($config) || empty($config['uri'])) {
        $seo_uri = 'download';
    } else {
        $seo_uri = rawurlencode($config['uri']);
    }
    $viewer_type   = WpfdBase::loadValue($params, 'use_google_viewer', 'lightbox');
    $perlink       = get_option('permalink_structure');
    $rewrite_rules = get_option('rewrite_rules');

    if (empty($config) || !isset($config['rmdownloadext'])) {
        $rmdownloadext = false;
    } else {
        $rmdownloadext = ((int) $config['rmdownloadext'] === 1) ? true : false;
    }

    $extension_list   = 'png,jpg,pdf,ppt,pptx,doc,docx,xls,xlsx,dxf,ps,eps,xps,psd,tif,tiff,bmp,svg,pages,ai,dxf,ttf,txt,mp3,mp4';
    $extension_viewer = explode(',', WpfdBase::loadValue($params, 'extension_viewer', $extension_list));
    $extension_viewer = array_map('trim', $extension_viewer);
    if (is_array($files)) {
        $filex = array();
        foreach ($files as $file) {
            if (!empty($rewrite_rules)) {
                if (strpos($perlink, 'index.php')) {
                    $linkdownload       = get_site_url() . '/index.php/' . $seo_uri . '/' . $termId . '/';
                    $linkdownload       .= $googleDriveCategoryTitle . '/' . $file->id . '/' . $file->post_title;
                    $file->linkdownload = $linkdownload;
                } else {
                    $linkdownload       = get_site_url() . '/' . $seo_uri . '/' . $termId . '/';
                    $linkdownload       .= $googleDriveCategoryTitle . '/' . $file->id . '/' . $file->post_title;
                    $file->linkdownload = $linkdownload;
                }
                if ($file->ext && !$rmdownloadext) {
                    $file->linkdownload .= '.' . $file->ext;
                }
            } else {
                $linkdownload       = admin_url('admin-ajax.php') . '?juwpfisadmin=false&action=wpfd&task=file.download';
                $linkdownload       .= '&wpfd_category_id=' . $termId . '&wpfd_file_id=' . $file->id;
                $file->linkdownload = $linkdownload;
            }
            $remote_url = (isset($file_meta) && isset($file_meta['remote_url']) && ((int) $file_meta['remote_url'] === 1)) ? true : false;
            if ($viewer_type !== 'no' &&
                in_array($file->ext, $extension_viewer) &&
                ($remote_url === false)) {
                $file->viewer_type = $viewer_type;
                $file->viewerlink  = WpfdHelperFile::isMediaFile($file->ext) ?
                    WpfdHelperFile::getMediaViewerUrl($file->id, $termId, $file->ext)
                    : WpfdHelperFile::getViewerUrl($file->id, $termId, $token);
            }
            $file->ggterm_id = $termId;
            $file->catid     = $termId;
            $filex[] = apply_filters('wpfda_file_info', $file, 'googleDrive');
        }
        return $filex;
    }

    return $files;
}
